import json
import operator
import tempfile

import yaml

from ansible.inventory import Inventory
from ansible.parsing.dataloader import DataLoader
from ansible.playbook import Playbook
from ansible.vars import VariableManager

from cloudmesh_client.shell.console import Console
from cloudmesh_client.common.Shell import Subprocess


class AnsiblePlaybook(object):

    def __init__(self, inventory, roles=None, path=None, username=None, become=False,
                 become_user=None, forks=None, modifyKnownHosts=True,
                 extra_vars=None, subprocess_kwargs=None):
        self._inventory = inventory
        self._roles = roles or []
        self._path = path
        self._username = username
        self._become = become
        self._become_user = become_user
        self._forks = forks
        self._modifyKnownHosts = modifyKnownHosts
        self._extra_vars = extra_vars
        self._subprocess_kwargs = subprocess_kwargs or dict()

        ################################################################ rest of setup

        self._inventory_file = tempfile.NamedTemporaryFile()
        self._playbook_file  = tempfile.NamedTemporaryFile()

        # write inventory
        self._inventory_file.write(self._inventory.ini())
        self._inventory_file.flush()

        Console.debug_msg('Inventory:\n' + self._inventory.ini())

        # write playbook
        playbook = [{
            'name': role.name,
            'hosts': role.hosts,
            'become': role.become,
            'roles': [dict(role=role.path,
                           **role.variables)]}
                    for role in roles]
        yml = yaml.dump(playbook)
        self._playbook_file.write(yml)
        self._playbook_file.flush()

        Console.debug_msg('Playbook:\n' + yml)

    def run(self):
        return self()

    def __call__(self):

        cmd_builder_monoid = [
            ['ansible-playbook'],

            ['--inventory', self._inventory_file.name]
            if self._inventory else [],

            ['--user', self._username]
            if self._username else [],

            ['--become']
            if self._become else [],

            ['--become-user', self._become_user]
            if self._become_user else [],

            ['--forks', str(self._forks)]
            if self._forks else [],

            ['--ssh-extra-args', '-o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no']
            if not self._modifyKnownHosts else []

            ['--extra-vars', json.dumps(self._extra_vars)]
            if self._extra_vars else [],

            [self._playbook_file.name],
        ]

        cmd = reduce(operator.concat, cmd_builder_monoid)
        Subprocess(cmd, **self._subprocess_kwargs)


        ################################################################



if __name__ == '__main__':
    loader = DataLoader()
    variable_manager = VariableManager()
    playbook = Playbook(loader)
    Inventory(loader, variable_manager, host_list={'a': ['foo', 'bar'], 'b': ['foo', 'baz']})




import os.path

from cloudmesh_client.platform.virtual_cluster.cluster import Cluster
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.default import Default
from cloudmesh_client.deployer.ansible.inventory import Inventory
from cloudmesh_client.deployer.ansible.role import AnsibleRole
from cloudmesh_client.deployer.ansible.playbook import AnsiblePlaybook
from cloudmesh_client.shell.command import (CloudPluginCommand, PluginCommand,
    command)
from cloudmesh_client.shell.console import Console


class AnsibleCommand(object):

    def playbook(self, cluster=None, path=None):
        Console.debug_msg('NotImplementedError')


    def role(self, cluster=None, uris=None, hostsPattern='all',
             become=True, username=None, modifyKnownHosts=True):
        """Deploy a role to all the nodes in the cluster

        :param str cluster: cluster name (default is the active cluster)
        :param str path: path to the role (default is current working directory)
        :param str username: login username of the nodes (overrides autodetected value)
        :param modifyKnownHosts: whether to update ~/.ssh/known_hosts
        """

        uris = uris or []
        assert len(uris) > 0, uris

        cluster = Cluster.from_name(cluster or Default.cluster)
        inventory = Inventory.from_cluster(cluster)
        roles = [AnsibleRole(uri, become=become, hosts=hostsPattern) for uri in uris]
        play = AnsiblePlaybook(
            roles=roles,
            inventory=inventory,
            username=username,
            modifyKnownHosts=modifyKnownHosts,
            subprocess_kwargs=dict(stdout=None, stderr=None),
        )

        play.run()



class DeployCommand(PluginCommand, CloudPluginCommand):
    topics = {'deploy': 'cluster'}

    def __init__(self, context):
        Console.debug_msg("init command deploy")

    @command
    def do_deploy(self, args, arguments):
        """
        ::
            Usage:
              deploy ansible role [-c CLUSTER] [-u NAME] [-N] [-H HOST] [-b] URI...

            Commands:

              ansible                        Ansible deployment

            Ansible Commands:
              playbook                       Deploy a pre-prepared playbook
              role                           Deploy a role to all nodes

            Arguments:

              CLUSTER                        Cluster name to deploy to
              NAME                           Alphanumeric name
              COUNT                          Integer > 0
              URI                            Location of the item as a uri
              HOST                           Host matching pattern

            Options:

              -c --cluster=CLUSTER           Cluster name to operate on (defaults to active)
              -N --no-modify-known-hosts     Don't let ssh update ~/.ssh/known_hosts
              -u --username=NAME             Username of the nodes to manage
              -H HOST --hosts=HOST           Host matching pattern [default: all]
              -b --no-become                 Don't become privileged user
        """

        arguments = dotdict(arguments)
        from pprint import pprint
        pprint(arguments)

        if arguments.ansible and arguments.role:
            ansible = AnsibleCommand()
            ansible.role(
                cluster = arguments.CLUSTER,
                hostsPattern = arguments['--hosts'],
                uris = arguments.URI,
                become = not arguments['--no-become'],
                username = arguments['--username'],
                modifyKnownHosts = not arguments['--no-modify-known-hosts'],
            )


if __name__ == '__main__':
    from docopt import docopt
    print(docopt(DeployCommand.do_deploy.__doc__))


    # import sys
    # rolepath = sys.argv[1]

    # c = AnsibleCommand()
    # c.role(path=rolepath)



""" run with

python setup.py install; nosetests -v --nocapture  tests/cm_cloud/test_image.py:Test_image.test_001

nosetests -v --nocapture tests/test_image.py

or

nosetests -v tests/test_image.py

"""
from __future__ import print_function

from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import HEADING
from cloudmesh_client.common.util import banner
from cloudmesh_client.default import Default
from cloudmesh_client.cloud.image import Image
from pprint import pprint

# noinspection PyPep8Naming
class Test_image:
    """
        This class tests the ImageCommand
    """

    data = dotdict({
        "cloud": Default.cloud,
        "wrong_cloud": "no_cloud",
        "format": "json"
    })

    def run(self, command):
        command = command.format(**self.data)
        banner(command, c="-")
        print(command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return result

    def setup(self):
        pass

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_001(self):
        HEADING("test image refresh")
        result = self.run("cm image refresh --cloud={cloud}")
        assert "ok." in result

    def test_002(self):
        HEADING("test image refresh fail")
        result = self.run("cm image refresh --cloud={wrong_cloud}")
        assert "failed" in result

    def test_003(self):
        HEADING("test image list")
        result = self.run("cm image list --cloud={cloud}")
        assert "description" in result

    def test_004(self):
        HEADING("test image list fail")
        result = self.run("cm image list --cloud={wrong_cloud}")
        assert "failed" in result

    def test_005(self):
        HEADING("test image list ID")
        result = self.run("cm image list --cloud={cloud} --format={format}")
        assert "Ubuntu" in result

    def test_006(self):
        HEADING("test image list ID fail")
        result = self.run("cm image list i --cloud={wrong_cloud}")
        assert "failed" in result

    def test_007(self):
        HEADING("test image username guess")

        result = Image.guess_username("Ubuntu-image")
        print(result)
        assert result == "ubuntu"

        result = Image.guess_username("wily-image")
        print(result)
        assert result == "ubuntu"

        result = Image.guess_username("fedora-image")
        print(result)
        assert result == "root"

        result = Image.guess_username("image", description="image with wily")
        print(result)
        assert result == "ubuntu"

    def test_008(self):
        HEADING("test image get username")

        # this test only works on chameleon
        if not Default.cloud in ["cm","chameleon"]:
            assert True

        result = self.run("cm image list --refresh")

        Image.set_username(name="CC-Ubuntu14.04", cloud=Default.cloud, username="undefined")
        result = Image.get_username("CC-Ubuntu14.04", Default.cloud)
        print ('Username:', result)
        assert "undefined" in result

        Image.set_username(name="CC-Ubuntu14.04", cloud=Default.cloud, username=None)
        result = Image.get_username("CC-Ubuntu14.04", Default.cloud, guess=True)
        print("Username:", result)


        assert "cc" in result


    def test_009(self):
        HEADING("test image get username")

        # this test only works on chameleon
        if not Default.cloud in ["cm", "chameleon"]:
            assert True

        login  = Image.get_username("CC-Ubuntu14.04", Default.cloud, guess=True)

        print ("LOGIN", login)

        Image.set_username(name="CC-Ubuntu14.04", cloud=Default.cloud, username=login)

        result  = Image.get_username("CC-Ubuntu14.04", Default.cloud)


        print(result)




from future.utils import with_metaclass
from cloudmesh_client.common.Printer import Printer

class ListResource(object):
    def _init__(self,
                category=None,
                kind=None,
                order=None,
                header=None):
        self.category = category
        self.kind = kind

    def info(cls, **kwargs):
        raise NotImplementedError()

    def list(cls, **kwargs):
        raise NotImplementedError()

    def clear(cls, **kwargs):
        raise NotImplementedError()

    def refresh(cls, **kwargs):
        raise NotImplementedError()

    @classmethod
    def list(cls,
             category=None,
             order=None,
             output='table'):
        """
        lists the default values in the specified format.
        TODO: This method has a bug as it uses format and output,
        only one should be used.

        :param category: the category of the default value. If general is used
                      it is a special category that is used for global values.
        :param order: The order in which the attributes are returned
        :param output: The output format. json, table, yaml, dict, csv
        :return:
        """

        # if order is None:
        #    # (order, header) = CloudProvider(category).get_attributes("default")
        try:
            if category is None:
                d = cls.cm.all("default")
            else:
                d = cls.cm.find('default', category=category)
            return (Printer.dict_printer(d,
                                         order=order,
                                         output=output))
        except:
            return None

    #
    # GENERAL SETTER AND GETTER METHOD
    #

    @classmethod
    def set(cls, key, value, category=None, user=None):
        """
        sets the default value for a given category
        :param key: the dictionary key of the value to store it at.
        :param value: the value
        :param category: the name of the category
        :param user: the username to store this default value at.
        :return:
        """

        try:

            o = cls.get_object(key, category)
            me = cls.cm.user or user
            if o is None:
                o = cls.cm.db_obj_dict('default',
                                       name=key,
                                       value=value,
                                       category=category,
                                       user=me)
                cls.cm.add_obj(o)
            else:
                o.value = value
                cls.cm.add_from_path(o)
                # cls.cm.update(o)
            cls.cm.save()
        except:
            return None

    @classmethod
    def get_object(cls, key, category="general"):
        """
        returns the first object that matches the key in the Default
        database.

        :param key: The dictionary key
        :param category: The category
        :return:
        """

        try:
            o = cls.cm.find(category=category,
                            kind='default',
                            output='object',
                            name=key,
                            scope='first')
            return o
        except Exception:
            return None

    @classmethod
    def get(cls, key, category="general"):
        """
        returns the value of the first objects matching the key
        with the given category.

        :param key: The dictionary key
        :param category: The category
        :return:
        """

        o = cls.cm.find(category=category,
                        kind='default',
                        output='dict',
                        scope='first',
                        name=key)
        if o is not None:
            return o['value']
        else:
            return None

    @classmethod
    def delete(cls, key, category):
        #
        # TODO: this is wrong implemented,
        #
        try:
            o = Default.get_object(key, category)
            if o is not None:
                cls.cm.delete(o)
                return "Deletion. ok."
            else:
                return None
        except:
            return None

    @classmethod
    def clear(cls):
        """
        deletes all default values in the database.
        :return:
        """
        try:
            d = cls.cm.all('default')
            for item in d:
                name = d[item]["name"]
                cls.cm.delete_by_name('default', name)
            cls.cm.save()
        except:
            return None

    '''
    # @abstractmethod
    @classmethod
    def get(cls, **kwargs):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def copy(cls, **kwargs):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def merge(cls, **kwargs):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __delitem__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __add__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __iadd__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __isub__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __concat__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __contains__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __sub__(cls, a, b):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __getitem__(cls, item):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def __setitem__(cls, item):
        raise NotImplementedError()

    # @abstractmethod
    @classmethod
    def count(self):
        raise NotImplementedError()
    '''



from __future__ import print_function

from cloudmesh_client.shell.console import Console
from cloudmesh_client.common.Printer import Printer
from cloudmesh_client.db import CloudmeshDatabase
from cloudmesh_client.cloud.iaas.CloudProvider import CloudProvider
from cloudmesh_client.cloud.ListResource import ListResource


class Flavor(ListResource):
    cm = CloudmeshDatabase()

    @classmethod
    def refresh(cls, cloud):
        """
        This method would refresh the flavor list by first clearing
        the database, then inserting new data
        :param cloud: the cloud name
        """

        return cls.cm.refresh('flavor', cloud)

    @classmethod
    def list(cls, cloud, live=False, format="table"):
        """
        This method lists all flavors of the cloud
        :param cloud: the cloud name
        """

        try:

            if live:
                cls.refresh(cloud)

            elements = cls.cm.find(kind="flavor", category=cloud)

            # pprint(elements)

            (order, header) = CloudProvider(cloud).get_attributes("flavor")

            return Printer.write(elements,
                                 order=order,
                                 header=header,
                                 output=format)
        except Exception as ex:
            Console.error(ex.message)

    @classmethod
    def details(cls, cloud, id, live=False, format="table"):
        if live:
            cls.refresh(cloud)

        return CloudProvider(cloud).details('flavor', cloud, id, format)





from cloudmesh_client.common.Printer import Printer
import requests
from cloudmesh_client.cloud.ListResource import ListResource
from cloudmesh_client.cloud.iaas.CloudProvider import CloudProvider

requests.packages.urllib3.disable_warnings()


class Quota(ListResource):

    @classmethod
    def list(cls, cloud, tenant, output="table"):
        try:

            provider = CloudProvider(cloud).provider

            result = provider.list_quota(cloud)

            (order, header) = CloudProvider(cloud).get_attributes("quota")

            return Printer.attribute(result,
                                     header=header,
                                     output=output)
        except Exception as e:
            import sys
            print(sys.exc_info()[0])
            return e



from __future__ import print_function
from cloudmesh_client.shell.command import command
from cloudmesh_client.shell.console import Console
from cloudmesh_client.default import Default
from cloudmesh_client.cloud.secgroup import SecGroup
from cloudmesh_client.shell.command import PluginCommand, CloudPluginCommand
from cloudmesh_client.common.dotdict import dotdict
from pprint import pprint
from cloudmesh_client.common.Printer import Printer

class SecgroupCommand(PluginCommand, CloudPluginCommand):
    topics = {"secgroup": "security"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command secgroup")

    def _delete(self, arg):
        if arg.cloud is None:
            Console.ok("Delete secgroup {GROUP}".format(**arg))
            SecGroup.delete(group=arg.GROUP)
        else:
            Console.ok("Delete secgroup {cloud}:{GROUP}".format(**arg))
            result = SecGroup.delete_secgroup(name=arg.GROUP, cloud=arg.cloud)
            if result is not None:
                Console.ok("Security Group={GROUP} in cloud={cloud} deleted successfully."
                           .format(**arg))
            else:
                Console.error("Failed to delete Security Group={GROUP} in cloud={cloud}"
                              .format(**arg))


    # noinspection PyUnusedLocal
    @command
    def do_secgroup(self, args, arguments):
        """
        ::

            Usage:
                secgroup list [--format=FORMAT]
                secgroup list --cloud=CLOUD [--format=FORMAT]
                secgroup list GROUP [--format=FORMAT]
                secgroup add GROUP RULE FROMPORT TOPORT PROTOCOL CIDR
                secgroup delete GROUP [--cloud=CLOUD]
                secgroup delete GROUP RULE
                secgroup upload [GROUP] [--cloud=CLOUD]

            Options:
                --format=FORMAT Specify output format, in one of the following:
                                table, csv, json, yaml, dict. The default value
                                is 'table'.
                --cloud=CLOUD   Name of the IaaS cloud e.g. kilo,chameleoon.
                                The clouds are defined in the yaml file.
                                If the name "all" is used for the cloud all
                                clouds will be selected.

            Arguments:
                RULE          The security group rule name
                GROUP         The label/name of the security group
                FROMPORT      Staring port of the rule, e.g. 22
                TOPORT        Ending port of the rule, e.g. 22
                PROTOCOL      Protocol applied, e.g. TCP,UDP,ICMP
                CIDR          IP address range in CIDR format, e.g.,
                              129.79.0.0/16

            Description:
                security_group command provides list/add/delete
                security_groups for a tenant of a cloud, as well as
                list/add/delete of rules for a security group from a
                specified cloud and tenant.


            Examples:
                secgroup list
                secgroup list --cloud=kilo
                secgroup add my_new_group webapp 8080 8080 tcp 0.0.0.0/0
                seggroup delete my_group my_rule
                secgroup delete my_unused_group --cloud=kilo
                secgroup upload --cloud=kilo

            Description:

                Security groups are first assembled in a local database.
                Once they are defined they can be added to the clouds.

                secgroup list [--format=FORMAT]
                    lists all security groups and rules in the database

                secgroup list GROUP [--format=FORMAT]
                    lists a given security group and its rules defined
                    locally in the database

                secgroup list --cloud=CLOUD [--format=FORMAT]
                    lists the security groups and rules on the specified clouds.

                secgroup add GROUP RULE FROMPORT TOPORT PROTOCOL CIDR
                    adds a security rule with the given group and the details
                    of the security ruls

                secgroup delete GROUP [--cloud=CLOUD]
                    Deletes a security group from the local database. To make
                    the change on the remote cloud, using the 'upload' command
                    afterwards.
                    If the --cloud parameter is specified, the change would be
                    made directly on the specified cloud

                secgroup delete GROUP RULE
                    deletes the given rule from the group. To make this change
                    on the remote cloud, using 'upload' command.

                secgroup upload [GROUP] [--cloud=CLOUD...]
                    uploads a given group to the given cloud. If the cloud is
                    not specified the default cloud is used.
                    If the parameter for cloud is "all" the rules and groups
                    will be uploaded to all active clouds.
                    This will synchronize the changes (add/delete on security
                    groups, rules) made locally to the remote cloud(s).

        """

        arg = dotdict(arguments)
        if arguments["--cloud"] is not None:
            is_cloud = True
            arg.cloud = arguments["--cloud"] or Default.cloud
        else:
            is_cloud = False

        arg.FORMAT = arguments["--format"] or 'table'

        # list all security-groups in cloud

        if arguments["list"]:

            if not is_cloud:

                if arg.RULE is None:
                    print(SecGroup.list(group=arg.GROUP, name=arg.RULE, output=arg.FORMAT))
                else:
                    print(SecGroup.list(group=arg.GROUP, output=arg.FORMAT))

            else:

                print(SecGroup.list(category=arg.cloud, output=arg.FORMAT))

        elif arguments["add"]:

            try:
                SecGroup.add_rule_to_db(
                    name=arg.RULE,
                    group=arg.GROUP,
                    from_port=arg.FROMPORT,
                    to_port=arg.TOPORT,
                    protocol=arg.PROTOCOL,
                    cidr=arg.CIDR)
            except:
                Console.error("Problem adding security group to db")


        # Delete a security-group
        elif arguments["delete"]:
            if arg["RULE"] is not None:
                SecGroup.delete_rule_from_db(group=arg["GROUP"],
                                             name=arg["RULE"])
            else:
                self._delete(arg)

        elif arguments["upload"]:

            # rewrite the _delete
            #
            # upload does not implicitly deleting a secgroup anymore
            # instead, it will check and update the rules only
            # self._delete(arg)
            SecGroup.upload(cloud=arg.cloud, group=arg.GROUP)

        return ""


'''
    # Create a security-group
    elif arguments["create"]:

        # If default not set, terminate
        if arg.cloud is not None:
            Console.error("Default cloud not set.")
            return

        # Create returns uuid of created sec-group
        uuid = SecGroup.create(arg.label, arg.cloud)

        if uuid:
            Console.ok("Created a new security group={label} with UUID={uuid}"
                       .format(**arg))
        else:
            Console.error("Exiting!")
        return ""
'''


""" run with

python setup.py install; nosetests -v --nocapture tests/cm_basic/test_model.py:Test_model.test_001

nosetests -v --nocapture tests/cm_basic/test_model.py

or

nosetests -v tests/cm_basic/test_model.py

"""
from __future__ import print_function

from pprint import pprint

from cloudmesh_client.common.FlatDict import FlatDict
from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import HEADING
from cloudmesh_client.common.util import banner
from cloudmesh_client.db import VM_OPENSTACK, VM_LIBCLOUD
from cloudmesh_client.db.CloudmeshDatabase import CloudmeshDatabase
from cloudmesh_client.default import Default
from cloudmesh_client.shell.console import Console
from cloudmesh_client.common.util import banner


# import cloudmesh_client
# cloudmesh_client.init()


# noinspection PyPep8Naming
class Test_cloud_model(object):
    cm = CloudmeshDatabase()

    data = dotdict({
        "cloud": Default.cloud,
    })

    def run(self, command):
        command = command.format(**self.data)
        banner(command, c="-")
        print(command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return str(result)

    def setup(self):
        self.d = {
            'name': 'vm1',
            'cloud': 'india',
            'update': '2015-06-18 22:11:48 UTC',
            'user': 'gregor',
            'extra': {'created': '2015-05-21T20:37:10Z',
                      'metadata': {'base_image_ref': '398746398798372493287',
                                   'description': None,
                                   'image_location': 'snapshot',
                                   'image_state': 'available',
                                   'image_type': 'snapshot',
                                   'instance_type_ephemeral_gb': '0',
                                   'instance_type_flavorid': '3',
                                   'instance_type_id': '1',
                                   'instance_type_memory_mb': '4096',
                                   'instance_type_name': 'm1.medium',
                                   'instance_type_root_gb': '40',
                                   'instance_type_rxtx_factor': '1.0',
                                   'instance_type_swap': '0',
                                   'instance_type_vcpus': '2',
                                   'instance_uuid': '386473678463876387',
                                   'kernel_id': None,
                                   'network_allocated': 'True',
                                   'owner_id': '36487264932876984723649',
                                   'ramdisk_id': None,
                                   'user_id': '762387463827463278649837'},
                      'minDisk': 40,
                      'minRam': 0,
                      'progress': 100,
                      'serverId': 'yiuksajhlkjahl',
                      'status': 'ACTIVE',
                      'updated': '2015-05-27T02:11:48Z'},
            'id': '39276498376478936247832687',
            'name': 'VM with Cloudmesh Configured Completely'
        }

        self.vm = {
            'extra': {'access_ip': '',
                      'availability_zone': 'nova',
                      'config_drive': '',
                      'created': '2015-06-19T00:06:58Z',
                      'disk_config': 'MANUAL',
                      'flavorId': '1',
                      'hostId': '',
                      'imageId': 'abcd',
                      'key_name': None,
                      'metadata': {},
                      'password': '********',
                      'tenantId': '1234',
                      'updated': '2015-06-19T00:06:58Z',
                      'uri': 'http://i5r.idp.iu.futuregrid.org/v2/1234/servers/abcd'},
            'id': '67f6bsf67a6b',
            'image': None,
            'name': 'vm2',
            'private_ips': [],
            'public_ips': [],
            'size': None,
            'state': 3
        }

        self.d = FlatDict(self.vm)

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_000(self):
        result = self.run("cm refresh off")
        print

    def test_001(self):
        HEADING("check the model")

        self.cm.clean()

        d = self.d

        banner("VM Data", c='-')
        pprint(d.__dict__)
        assert d['extra__imageId'] == 'abcd'

        banner("Add VM", c='-')

        name = "vm1"
        uuid = d.id

        vm = VM_OPENSTACK(name=name,
                          user="test",
                          category=self.data.cloud,
                          **d)
        print(vm)
        banner("VM added", c='-')

        self.cm.add(vm)

        banner("Add VM", c='-')

        name = "vm2"
        uuid = d.id

        vm = VM_OPENSTACK(name=name,
                          user="test",
                          category=self.data.cloud,
                          **d)
        print(vm)
        banner("VM added", c='-')

        # pprint(vm.__dict__)
        # vm.bla = "bla"
        self.cm.add(vm)
        # self.cm.save()

        banner("Get VM from Database", c='-')

        vms = self.cm.find(kind="vm", name=name, output='dict')
        pprint(vms)
        assert len(vms) == 1
        assert vms[0]["name"] == name

    def test_002(self):
        HEADING("VM DB test")
        self.cm.clean()

        print("ADD TO OS ")

        d = {
            'user': "test",
            'category': self.data.cloud
        }
        for index in range(1, 6):
            name = "vm_" + str(index).zfill(3)
            banner(name)
            print("ADD", name)
            try:
                d['name'] = name
                d['uuid'] = "uuid_" + str(index)
                vm = VM_OPENSTACK(**d)
                pprint(vm.__dict__)
            except Exception as e:
                Console.error("issue adding vm", traceflag=True)
            self.cm.add(vm)
            print("VM added. ok")
        print("ADD TO LIBCLOUD ")
        for index in range(6, 11):
            name = "vm_" + str(index).zfill(3)
            banner(name)
            print("ADD", name)
            d['name'] = name
            d['uuid'] = "uuid_" + str(index)
            vm = VM_LIBCLOUD(**d)
            print("VM", vm.__dict__)
            self.cm.add(vm)
        self.cm.save()

        result = self.run("cm refresh off")
        print(result)

        vms = self.cm.find(kind="vm", scope="all", output='dict')
        pprint(vms)
        print (len(vms))
        assert len(vms) == 10


    def test_003(self):
        HEADING("find vm tables")

        print("---------")
        all_tables = self.cm.tables
        for t in all_tables:
            print(t.__tablename__, t.__kind__)

        print(all_tables)
        assert 'DEFAULT' in str(all_tables)

        # print ("---------")
        # vm_tables = self.cm.tables(kind="vm")
        # for t in vm_tables:
        #     print (t.__tablename__, t.__kind__)
        # assert len(vm_tables) == 2

    def test_004(self):
        HEADING("find vm tables")

        print("-------------")
        vm = self.cm.find(kind="vm", name="vm_001", scope='first')
        pprint(vm)
        assert vm.name == 'vm_001'

        print("-------------")
        vm = self.cm.find(kind="vm", name="vm_006", scope='first')
        pprint(vm)
        assert vm.name == 'vm_006'

        print("-------------")
        vms = self.cm.find(kind="vm", scope="all")
        pprint(vms)
        print(len(vms))
        assert len(vms) == 10



from __future__ import print_function
from cloudmesh_client.common.ConfigDict import ConfigDict

from cloudmesh_client.common.todo import TODO
# add imports for other cloud providers in future
from cloudmesh_client.shell.console import Console
from cloudmesh_client.cloud.ListResource import ListResource
from cloudmesh_client.common.Printer import Printer
from cloudmesh_client.db import CloudmeshDatabase
from cloudmesh_client.cloud.iaas.CloudProvider import CloudProvider
from cloudmesh_client.common.Error import Error
from uuid import UUID
from cloudmesh_client.common.dotdict import dotdict
from builtins import input
from pprint import pprint
from cloudmesh_client.cloud.network import Network
from cloudmesh_client.default import (Names, Default)
import traceback


# noinspection PyPep8Naming
class Vm(ListResource):
    cm = CloudmeshDatabase()

    @classmethod
    def generate_vm_name(cls, prefix=None, fill=3):
        return Default.generate_name(Names.VM_COUNTER,
                                     display_name=Default.user,
                                     prefix=prefix,
                                     fill=fill)

    @classmethod
    def uuid(cls, name, category=None):
        vm = cls.get(name, category=category)
        if vm is None:
            return None

        return vm.uuid

    @classmethod
    def get(cls, key, category=None):
        """
        returns the value of the first objects matching the key
        with the given category.

        :param key: The dictionary key
        :param category: The category
        :return:
        """

        if category is None:
            o = cls.cm.find(kind='vm',
                            output='dict',
                            scope='first',
                            name=key)

        else:
            o = cls.cm.find(category=category,
                            kind='vm',
                            output='dict',
                            scope='first',
                            name=key)
        return o

    @classmethod
    def construct_ip_dict(cls, ip_addr, name=None):
        # TODO kilo cloud as defualt should be avoided
        if name is None:
            Console.error("cloud name not set")
            return None
        try:
            d = ConfigDict("cloudmesh.yaml")
            cloud_details = d["cloudmesh"]["clouds"][name]

            # Handle Openstack Specific Output
            if cloud_details["cm_type"] == "openstack":
                ipaddr = {}
                for network in ip_addr:
                    index = 0
                    for ip in ip_addr[network]:
                        ipaddr[index] = {}
                        ipaddr[index]["network"] = network
                        ipaddr[index]["version"] = ip["version"]
                        ipaddr[index]["addr"] = ip["addr"]
                        index += 1
                return ipaddr

            # Handle EC2 Specific Output
            if cloud_details["cm_type"] == "ec2":
                # Console.TODO("ec2 ip dict yet to be implemented")
                # TODO.implement()
                # :type str: ip_addr
                index = 0
                ipaddr = {}
                ipaddr[index] = {}
                ipaddr[index]['addr'] = ip_addr
                return ipaddr

            # Handle Azure Specific Output
            if cloud_details["cm_type"] == "azure":
                index = 0
                ipaddr = {}
                for ip in ip_addr:
                    ipaddr[index] = {}
                    ipaddr[index]["network"] = ip
                    ipaddr[index]["version"] = 'ipv4'
                    ipaddr[index]["addr"] = ip
                    index += 1
                return ipaddr

        except Exception as e:
            Console.error("error in vm construct dict %s" % e, traceflag=True)

    @classmethod
    def isUuid(cls, name):
        try:
            UUID(name, version=4)
            return True
        except ValueError:
            return False

    @classmethod
    def boot(cls, **kwargs):

        arg = dotdict(kwargs)

        for a in ["key", "name", "image", "flavor"]:
            if a not in kwargs:
                raise ValueError(a + " not in arguments to vm boot")

        conf = ConfigDict("cloudmesh.yaml")
        arg.user = arg.user or conf["cloudmesh"]["profile"]["user"]
        arg.group = arg.group or Default.group
        cloud_provider = CloudProvider(arg.cloud).provider

        if "nics" in arg:
            nics = arg.nics
        else:
            nics = None

        basic_dict = {
            "cloud": arg.cloud,
            "name": arg.name,
            "image": arg.image,
            "flavor": arg.flavor,
            "key": arg.key,
            "secgroup": [arg.secgroup],
            "nics": nics,
            "meta": {
                'kind': 'cloudmesh',
                'group': arg.group,
                'cluster': arg.get('cluster', None),
                'image': arg.image,
                'flavor': arg.flavor,
                'key': arg.key,
                'category': arg.cloud
            }
        }

        # Special case for Azure where certificate details needs to be added
        if arg.cloud == "azure":
            kwargs = dict()
            kwargs['kind'] = "key_azure"
            db_result = cls.cm.find(**kwargs)
            # pprint("Key DB results")
            key_result = None
            try:
                for key in db_result:
                    if key['name'] == arg.key:
                        pprint("Found the key")
                        key_result = key
                        break
                if key_result is not None:
                    new_dict_items = dict()
                    new_dict_items['cert_thumbprint'] = key_result['fingerprint']
                    new_dict_items['pub_key_path'] = key_result['key_path']
                    new_dict_items['cert_path'] = key_result['certificate']
                    new_dict_items['pfx_path'] = key_result['pfx_path']
                    basic_dict.update(new_dict_items)
                else:
                    pprint("None found in DB")
            except:
                traceback.print_exc()
                pprint("Exception while processing azure boot arguments")
        d = dotdict(basic_dict)

        Console.ok("Machine {name} is being booted on cloud {cloud} ..."
                   .format(**arg))

        print(Printer.attribute(d))

        vm = cloud_provider.boot_vm(**d)
        if vm is not None:
            cls.refresh(cloud=arg.cloud)

            try:
                # TODO: Repair db schema for vm_azure, vm_libcloud,
                # vm_openstack. The following set only works with
                # openstack, no libcloud, no azure

                cls.cm.set(d.name, "key", d.key, scope="first", kind="vm")
                cls.cm.set(d.name, "image", d.image, scope="first", kind="vm")
                cls.cm.set(d.name, "flavor", d.flavor, scope="first", kind="vm")
                cls.cm.set(d.name, "group", arg.group, scope="first", kind="vm")
                cls.cm.set(d.name, "user", arg.user, scope="first", kind="vm")
                cls.cm.set(d.name, 'username', arg.username, scope='first', kind='vm')
                cls.cm.set(d.name, 'cluster', arg.cluster, scope='first', kind='vm')

            except:
                # cm.set error is identified as a warning, not an error
                import sys
                Console.warning("cls.cm.set error: %s" % (sys.exc_info()[0]))

        # update group and key
        #
        # cls.cm.update("vm", name=data.name)

        return vm

    @classmethod
    def start(cls, **kwargs):
        arg = dotdict(kwargs)
        cloud_provider = CloudProvider(arg.cloud).provider
        for server in kwargs["servers"]:
            cloud_provider.start_vm(server)
            Console.ok("Machine {:} is being started on {:} Cloud..."
                       .format(server, cloud_provider.cloud))

            # Explicit refresh called after VM start, to update db.
            # cls.refresh(cloud=kwargs["cloud"])

    @classmethod
    def stop(cls, **kwargs):
        arg = dotdict(kwargs)
        cloud_provider = CloudProvider(arg.cloud).provider
        for server in kwargs["servers"]:
            cloud_provider.stop_vm(server)
            Console.ok("Machine {:} is being stopped on {:} Cloud..."
                       .format(server, cloud_provider.cloud))

            # Explicit refresh called after VM stop, to update db.
            # cls.refresh(cloud=kwargs["cloud"])

    @classmethod
    def delete(cls, **kwargs):
        arg = dotdict(kwargs)

        force = kwargs.get("force", Default.purge)

        if "cloud" in arg:
            cloud_provider = CloudProvider(arg.cloud).provider
            for server in kwargs["servers"]:
                vm = cls.cm.find(name=server, kind="vm",
                                 cloud=arg.cloud, scope="first")
                #vm_by_id = cls.cm.find(cm_id=server, kind="vm", cloud=arg.cloud, scope="first")
                #print (vm)
                #print(vm_by_id)
                #vm = vm or vm_by_id
                if vm:
                    provider = vm["provider"]
                    cloud = vm["category"]

                    # If server has a floating ip associated, release it
                    server_dict = Network.get_instance_dict(
                        cloudname=arg.cloud,
                        instance_id=server)
                    floating_ip = server_dict["floating_ip"]
                    if floating_ip is not None:
                        Network.disassociate_floating_ip(
                            cloudname=arg.cloud,
                            instance_name=server,
                            floating_ip=floating_ip)
                    cloud_provider.delete_vm(server)
                    if force:
                        cls.cm.delete(kind="vm",
                                      provider=provider,
                                      category=cloud,
                                      name=server)  # delete the record from db
                        Console.ok(
                            "VM record {:} is being deleted from the local database..."
                            .format(server))

                    else:
                        cls.cm.set(server, "status", "deleted",
                                   kind="vm", scope="first")

                    # Console.ok("VM {:} is being deleted on {:} cloud...".format(server, cloud_provider.cloud))
                else:
                    Console.error("VM {:} can not be found."
                                  .format(server), traceflag=False)
        else:

            clouds = set()
            for server in arg.servers:

                vm = cls.cm.find(kind="vm", name=server, scope="first")
                if vm:
                    cloud = vm["category"]
                    provider = vm["provider"]
                    cloud_provider = CloudProvider(cloud).provider
                    clouds.add(cloud)
                    cloud_provider.delete_vm(server)
                    if force:
                        cls.cm.delete(kind="vm",
                                      provider=provider,
                                      category=cloud,
                                      name=server)
                        Console.ok(
                            "VM record {:} is being deleted from the local database..."
                            .format(server))

                    else:
                        cls.cm.set(server, "status", "deleted",
                                   kind="vm", scope="first")

                    # Console.ok("VM {:} is being deleted on {:} cloud...".format(server, cloud))
                else:
                    Console.error("VM {:} can not be found."
                                  .format(server), traceflag=False)

    @classmethod
    def get_vms_by_name(cls, name, cloud):

        vm_data = cls.cm.find(kind="vm", name=name, category=cloud)
        if vm_data is None or len(vm_data) == 0:
            raise RuntimeError("VM data not found in database.")
        return vm_data

    @classmethod
    def get_vms_by_group(cls, name):

        group = cls.cm.find(kind="group", name=name)
        return group

    @classmethod
    def get_vm(cls, name):

        vm = cls.cm.find(kind="vm", name=name)

        return vm

    @classmethod
    def rename(cls, **kwargs):

        arg = dotdict(kwargs)

        cloud_provider = CloudProvider(kwargs["cloud"]).provider

        # Check for vms with duplicate names in DB.
        vms = cls.get_vms_by_name(name=arg.oldname, cloud=arg.cloud)

        if len(vms) > 1:
            users_choice = "y"

            if not arg.force:
                print("More than 1 vms found with the same name as {}."
                      .format(server))
                users_choice = input(
                    "Would you like to auto-order the new names? (y/n): ")

            if users_choice.strip() == "y":
                count = 1
                for index in vms:
                    count_new_name = "{0}{1}".format(arg.newname, count)
                    # print(vms[index])

                    cloud_provider.rename_vm(vms[index]["uuid"],
                                             count_new_name)

                    print(
                        "Machine {0} with UUID {1} renamed to {2} on {3} cloud"
                        .format(vms[index]["name"],
                                vms[index]["uuid"],
                                count_new_name,
                                cloud_provider.cloud))
                    count += 1
            elif users_choice.strip() == "n":
                cloud_provider.rename_vm(arg.oldname, arg.newname)
                print(
                    "Machine {0} renamed to {1} on {2} Cloud..."
                    .format(arg.oldname, arg.newname, cloud_provider.cloud))
            else:
                Console.error("Invalid Choice.")
                return
        else:
            cloud_provider.rename_vm(arg.oldname, arg.newname)
            print("Machine {0} renamed to {1} on {2} Cloud..."
                  .format(arg.oldname, arg.newname, cloud_provider.cloud))

        # Explicit refresh called after VM rename, to update db.
        cls.refresh(cloud=arg.cloud)

    @classmethod
    def info(cls, **kwargs):
        raise NotImplementedError()

    @classmethod
    def list(cls, **kwargs):
        """
        This method lists all VMs of the cloud
        """

        arg = dotdict(kwargs)
        if "name" in arg:
            arg.name = arg.name

        arg.output = arg.output or 'table'

        # pprint (kwargs)
        # prevent circular dependency
        def vm_groups(vm):
            """

            :param vm: name of the vm
            :return: a list of groups the vm is in
            """

            try:
                query = {
                    'kind': "group",
                    'provider': 'general',
                    "species": "vm",
                    "member": vm,
                    "scope": 'all',
                    "output": 'dict'
                }

                d = cls.cm.find(**query)
                groups_vm = set()
                if d is not None and len(d) > 0:
                    for vm in d:
                        groups_vm.add(vm['name'])
                return list(groups_vm)
            except Exception as ex:
                Console.error(ex.message)
            return []

        try:
            if "name" in arg and arg.name is not None:
                if cls.isUuid(arg.name):
                    elements = cls.cm.find(kind="vm",
                                           category=arg.category,
                                           uuid=arg.name)
                else:
                    elements = cls.cm.find(kind="vm",
                                           category=arg.category,
                                           label=arg.name)
            else:
                elements = cls.cm.find(kind="vm",
                                       category=arg.category)

            if elements is None or len(elements) == 0:
                return None

            for elem in elements:
                element = elem
                name = element["name"]
                groups = vm_groups(name)
                element["group"] = ','.join(groups)

            # print(elements)

            # order = ['id', 'uuid', 'name', 'cloud']
            (order, header) = CloudProvider(arg.category).get_attributes("vm")

            # order = None
            if "name" in arg and arg.name is not None:
                return Printer.attribute(elements[0],
                                         output=arg.output)
            else:
                return Printer.write(elements,
                                     order=order,
                                     output=arg.output)
        except Exception as ex:
            Console.error(ex.message)

    @classmethod
    def clear(cls, **kwargs):
        raise NotImplementedError()

    @classmethod
    def refresh(cls, **kwargs):
        # print("Inside refresh")

        refreshed = cls.cm.refresh("vm", kwargs["cloud"])

        # update counter
        vms = cls.cm.find(kind='vm')
        me  = Default.user
        for vm in vms:
            name = vm['name']
            if not name.startswith(me):
                continue

            number = name.split('-')[-1]

            try:
                # +1 as the stored counter is the next available counter
                new_counter = int(number) + 1
            except ValueError:
                # name is not formatted correctly, possibly due to not
                # being started using cloudmesh
                continue

            old_counter = Default.get_counter(Names.VM_COUNTER)
            counter = max(new_counter, old_counter)
            Default.set_counter(Names.VM_COUNTER, counter)

        Console.debug_msg('Set counter ' + Names.VM_COUNTER + ' to ' +
                          str(Default.get_counter(Names.VM_COUNTER)))

        return refreshed

    @classmethod
    def status_from_cloud(cls, **kwargs):
        cloud_provider = CloudProvider(kwargs["cloud"]).provider
        vm = cloud_provider.get_vm(name=kwargs["name"])
        return vm["status"]

    @classmethod
    def set_login_user(cls, name=None, cloud=None, username=None):

        # cls.cm.set(name, "username", username, kind="vm", scope="first")

        vm = Vm.get(name, category=cloud)

        if vm is None:
            Console.error("VM could not be found", traceflag=False)
            return
        else:
            cls.cm.update(kind="vm",
                          provider=vm["provider"],
                          filter={'name': name},
                          update={"username": username}
                          )

    @classmethod
    def get_login_user(cls, name, cloud):
        print(name, cloud)

        Console.error("this method is wrong implemented")

        '''
        if cls.isUuid(name):
            uuid = name
        else:
            vm_data = cls.cm.find(kind="vm", category=cloud, label=name)
            if vm_data is None or len(vm_data) == 0:
                raise RuntimeError("VM with label {} not found in database.".format(name))
            uuid = list(vm_data.values())[0]["uuid"]

        # print(uuid)

        user_map_entry = cls.cm.find("VMUSERMAP", vm_uuid=uuid)

        # print(user_map_entry)

        if user_map_entry is None or len(user_map_entry) == 0:
            return None
        else:
            return list(user_map_entry.values())[0]["username"]
        '''

    @classmethod
    def get_vm_public_ip(cls, vm_name, cloud):
        """

        :param vm_name: Name of the VM instance whose Public IP has to be retrieved from the DB
        :param cloud: Libcloud supported Cloud provider name
        :return: Public IP as a list
        """
        public_ip_list = []
        vms = cls.get_vms_by_name(vm_name, cloud)
        keys = vms.keys()
        if keys is not None and len(keys) > 0:
            public_ip = vms[keys[0]]["public_ips"]
            if public_ip is not None and public_ip != "":
                public_ip_list.append(public_ip)
        return public_ip_list



from __future__ import print_function

import sys
from cloudmesh_client.platform.virtual_cluster.cluster import Cluster

from cloudmesh_client.cloud.image import Image
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.db.CloudmeshDatabase import CloudmeshDatabase
from cloudmesh_client.default import Default, Names
from cloudmesh_client.deployer.ansible.inventory import Inventory, Node
from cloudmesh_client.exc import (ClusterNameClashException,
    NoActiveClusterException, UnrecoverableErrorException)
from cloudmesh_client.shell.command import (CloudPluginCommand, PluginCommand,
    command)
from cloudmesh_client.shell.console import Console


db = CloudmeshDatabase


class Command(object):
    def create(self, clustername=None, cloud=None, count=1,
               username=None, image=None, flavor=None, key=None,
               secgroup=None, assignFloatingIP=True,
               activate=True):
            """Create a cluster.

            If values are `None`, they are automatically determined via
            defaults.

            :param str clustername: name of this cluster (generated if None)
            :param str cloud: cloud name
            :param int count: number of instances in the cluster
            :param str user: cloudmesh user
            :param str username: cloud image username
            :param str image: image name
            :param str flavor: instance flavor
            :param str key: key name
            :param str secgroup: security group name
            :param bool activate: activate this cluster after creation
            :returns: a cluster
            :rtype: :class:`Cluster`
            """

            clustername = clustername or Default.generate_name(Names.CLUSTER_COUNTER)
            cloud = cloud or Default.cloud
            username = username or Image.guess_username(image)
            image = image or Default.image
            flavor = flavor or Default.flavor
            key = key or Default.key
            secgroup = secgroup or Default.secgroup

            try:
                cluster = Cluster(
                    name=clustername,
                    count=count,
                    cloud=cloud,
                    username=username,
                    image=image,
                    flavor=flavor,
                    key=key,
                    secgroup=secgroup,
                    assignFloatingIP=assignFloatingIP,
                )
            except ClusterNameClashException as e:
                Console.error(str(e))
                raise UnrecoverableErrorException(str(e))

            cluster.create()
            Console.ok('Cluster {} created'.format(clustername))

            if activate:
                Default.set_cluster(clustername)
                Console.ok('Cluster {} is now active'.format(clustername))

            return cluster

    def list(self):
        """List the clusters created

        The currently active cluster is given as the first element in
        the tuple, while the remainder are listed in the second
        position.

        :returns: a pair: (active, [clusters])
        :rtype: :class:`tuple` (:class:`Cluster`, :class:`list` of :class:`Cluster`)
        :raises: :class:`NoActiveClusterException` if no cluster is active, meaning there are no clusters
        """

        activename = Default.active_cluster.name
        clusters = db.select(Cluster).all()
        active = filter(lambda c: c.name == activename, clusters)[0]
        clusters = filter(lambda c: c.name != activename, clusters)
        return active, clusters

    def delete(self, clusternames=None, force=False, all=False):
        """Delete clusters that have these names.

        If not specified, delete the active cluster.
        If there is no active cluster, delete the first cluster.

        :param list clusternames: list of cluster names to delete
        """

        if all:
            clusters = db.select(Cluster)
        else:

            clusternames = clusternames or [Default.cluster]
            clusters = [db.select(Cluster, name=name).one()
                        for name in clusternames]

        for cluster in clusters:
            Console.ok('Deleting cluster {}'.format(cluster.name))
            cluster.delete(force=force)
            Console.ok('Deleted cluster {}: {} nodes'
                       .format(cluster.name, cluster.count))

        remaining_clusters = db.select(Cluster).all()
        if remaining_clusters:
            name = remaining_clusters[-1].name
        else:
            name = None
        Default.set_cluster(name)
        Console.ok('Active cluster: {}'.format(name))

    def get(self, property, cluster=None):
        """Retrieve the property for a cluster/nodes in a cluster.

        If no cluster is specified, use the currently active cluster.

        :param str property: name of the property
        :param Cluster cluster: the cluster (default: currently active)
        :returns: a list of the values
        :rtype: :class:`list`

        """
        cluster = cluster or db.select(Cluster, name=Default.cluster).one()

        # getting from the cluster itself:
        if hasattr(cluster, property):
            return [getattr(cluster, property)]

        # assume it is a property of the instances
        else:
            values = [getattr(node, property) for node in cluster]
            return values

    def nodes(self, cluster=None):
        """Retrieve the nodes of a cluster

        If no cluster is specified, use the currently active cluster.

        :param Cluster cluster: the cluster (default: currently active)
        :returns: a list of instances
        :rtype: a VM instance

        """

        cluster = cluster or Default.active_cluster
        return cluster.list()


    def inventory(self, cluster=None, format=None, path=None):

        cluster = cluster or Default.active_cluster
        format = format or 'ansible'

        if format == 'ansible':

            inventory = Inventory.from_cluster(cluster)
            inv_ini = inventory.ini()

            if not path:
                print(inv_ini)
            else:
                with open(path, 'w') as fd:
                    fd.write(inv_ini)

            return inventory


class Cluster2Command(PluginCommand, CloudPluginCommand):
    topics = {'cluster2': 'cluster'}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command cluster2 ")

    @command
    def do_cluster2(self, args, arguments):
        """
        ::
            Usage:
              cluster2 create [-n NAME] [-c COUNT] [-C CLOUD] [-u NAME] [-i IMAGE] [-f FLAVOR] [-k KEY] [-s NAME] [-AI]
              cluster2 list
              cluster2 nodes [CLUSTER]
              cluster2 delete [--all] [--force] [NAME]...
              cluster2 get [-n NAME] PROPERTY
              cluster2 inventory [-F NAME] [-o PATH] [NAME]

            Commands:

              create     Create a cluster
              list       List the available clusters
              inventory  Obtain an inventory file
              delete     Delete clusters and associated instances
              get        Get properties of a cluster/nodes in a cluster

            Arguments:

              NAME                Alphanumeric name
              COUNT               Integer > 0
              PATH                Path to entry on the filesystem

            Options:

              -A --no-activate               Don't activate this cluster
              -I --no-floating-ip            Don't assign floating IPs
              -n NAME --name=NAME            Name of the cluster
              -c COUNT --count=COUNT         Number of nodes in the cluster
              -C NAME --cloud=NAME           Name of the cloud
              -u NAME --username=NAME        Name of the image login user
              -i NAME --image=NAME           Name of the image
              -f NAME --flavor=NAME          Name of the flavor
              -k NAME --key=NAME             Name of the key
              -s NAME --secgroup=NAME        NAME of the security group
              -F NAME --format=NAME          Name of the output format
              -o PATH --path=PATH            Output to this path
              --force
              --all

            Inventory File Format:

              ansible  [default]            Ansible-compatible inventory
        """

        arguments = dotdict(arguments)
        cmd = Command()

        if arguments.create:

            cmd.create(
                clustername=arguments['--name'],
                count=arguments['--count'] or 1,
                cloud=arguments['--cloud'] or Default.cloud,
                username=arguments['--username'],
                image=arguments['--image'] or Default.image,
                flavor=arguments['--flavor'] or Default.flavor,
                key=arguments['--key'] or Default.key,
                secgroup=arguments['--secgroup'] or Default.secgroup,
                assignFloatingIP=not arguments['--no-floating-ip'],
            )

        elif arguments.list:

            try:
                active, inactive = cmd.list()
            except NoActiveClusterException:
                return

            def show(cluster, isactive=False, stream=sys.stdout):
                if not cluster:
                    return
                if isactive:
                    stream.write('> ')
                else:
                    stream.write('  ')
                stream.write(cluster.name)
                stream.write('\n')
                stream.flush()

            show(active, isactive=True)
            map(show, inactive)

        elif arguments.nodes:

            cluster = db.select(Cluster, name=arguments.CLUSTER).one() \
                    if arguments.CLUSTER \
                    else None
            nodes = cmd.nodes(cluster=cluster)
            for node in nodes:
                print(node.name)

        elif arguments.delete:

            cmd.delete(
                arguments.NAME,
                force=arguments['--force'],
                all=arguments['--all']
            )

        elif arguments['get']:

            values = cmd.get(arguments['PROPERTY'], cluster=arguments['--name'])
            for v in values:
                print(v)

        elif arguments.inventory:
            cmd.inventory(
                cluster=arguments.NAME,
                format=arguments['--format'],
                path=arguments['--path'],
            )



if __name__ == '__main__':
    cmd = Cluster2Command()
    cmd.create(
        clustername='test3',
        image='CC-Ubuntu14.04',
        flavor='m1.medium',
        key='gambit',
        count=3,
    )





from __future__ import print_function
import base64
import hashlib
import struct
from os.path import basename

from cloudmesh_client.common.ConfigDict import Config


# noinspection PyBroadException
class SSHkey(object):
    def __init__(self, file_path=None, keyname=None):
        self.__key__ = None
        self.read(file_path, keyname)

    def get(self):
        return self.__key__

    def __str__(self):
        return self.__key__['key']

    def __repr__(self):
        return self.__key__['key']

    def read(self, file_path, keyname=None):
        self.__key__ = {}
        if file_path is not None:
            orig_path = file_path
            file_path = Config.path_expand(file_path)
            uri = 'file://{}'.format(file_path)
            self.__key__ = {
                'uri': uri,
                'path': orig_path,
                'string': open(file_path, "r").read().rstrip()
            }

            (self.__key__['type'],
             self.__key__['key'],
             self.__key__['comment']) = self._parse(self.__key__['string'])
            self.__key__['fingerprint'] = self._fingerprint(self.__key__['string'])
            # Workaround for multiple file entries in cloudmesh.yaml getting same name derived from file name (like id_rsa).
            # This caused the dict to have just 1 entry as the name is the key.
            # Change tracked in git issue #8
            if keyname is None:
                name = basename(file_path).replace(".pub", "").replace("id_", "")
            else:
                name = keyname

            self.__key__['name'] = name
            self.__key__['comment'] = self.__key__['comment']
            self.__key__['source'] = 'ssh'
        return self.__key__

    @property
    def fingerprint(self):
        return self.__key__['fingerprint']

    @property
    def key(self):
        return self.__key__['string']

    @property
    def type(self):
        return self.__key__['type']

    @property
    def comment(self):
        return self.__key__['comment']

    @classmethod
    def _fingerprint(cls, entirekey):
        """returns the fingerprint of a key.
        :param entirekey: the key
        :type entirekey: string
        """
        t, keystring, comment = cls._parse(entirekey)
        if keystring is not None:
            return cls._key_fingerprint(keystring)
        else:
            return ''

    @classmethod
    def _key_fingerprint(cls, key_string):
        """create the fingerprint form just the key.

        :param key_string: the key
        :type key_string: string
        """
        # key = base64.decodestring(key_string)
        # fp_plain = hashlib.md5(key).hexdigest()
        key_padding = key_string.strip() + '=' * (4 - len(key_string.strip()) % 4)
        key = base64.b64decode(key_padding.encode('ascii'))
        fp_plain = hashlib.md5(key).hexdigest()

        return ':'.join(a + b for a, b in zip(fp_plain[::2], fp_plain[1::2]))

    @classmethod
    def _parse(cls, keystring):
        """
        parse the keystring/keycontent into type,key,comment
        :param keystring: the content of a key in string format
        """
        # comment section could have a space too
        keysegments = keystring.split(" ", 2)
        keytype = keysegments[0]
        key = None
        comment = None
        if len(keysegments) > 1:
            key = keysegments[1]
            if len(keysegments) > 2:
                comment = keysegments[2]
        return keytype, key, comment

    def _validate(self, keytype, key):
        """reads the key string from a file. THIS FUNCTION HAS A BUG.

        :param key: either the name of  a file that contains the key, or the entire contents of such a file
        :param keytype: if 'file' the key is read form the file specified in key.
                        if 'string' the key is passed as a string in key
        """
        keystring = None
        if keytype.lower() == "file":
            try:
                keystring = open(key, "r").read()
            except:
                return False
        elif keytype.lower() == "string":
            keystring = key

        try:

            keytype, key_string, comment = self._parse(keystring)
            data = base64.decodestring(key_string)
            int_len = 4
            str_len = struct.unpack('>I', data[:int_len])[0]
            # this should return 7

            if data[int_len:int_len + str_len] == keytype:
                return True
        except Exception as e:
            # print(e)
            return False

    def _keyname_sanitation(self, username, keyname):
        keynamenew = "%s_%s" % (
            username, keyname.replace('.', '_').replace('@', '_'))
        return keynamenew


# unit testing


def main():
    from pprint import pprint

    sshkey = SSHkey("~/.ssh/id_rsa.pub")
    pprint(sshkey.key)
    print("Fingerprint:", sshkey.fingerprint)
    pprint(sshkey.__key__)
    print("sshkey", sshkey)
    print("str", str(sshkey))
    print(sshkey.type)
    print(sshkey.__key__['key'])
    print(sshkey.key)
    print(sshkey.comment)
    """
    key1 = "ssh-rsa abcdefg comment"
    key2 = "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDD+NswLi/zjz" + \
            "7Vf575eo9iWWku5m4nVSPMgP13JbKCTVKtavAXt8UPZTkYVWi" + \
            "USeXRqlf+EZM11U8Mq6C/P/ECJS868rn2KSwFosNPF0OOz8zm" + \
            "TvBQShtvBBBVd1kmZePxFGviZbKwe3z3iATLKE8h7pwcupqTi" + \
            "n9m3FhQRsGSF7YTFcGXv0ZqxFA2j9+Ix7SVbN5IYxxgwc+mxO" + \
            "zYIy1SKEAOPJQFXKkiXxNdLSzGgjkurhPAIns8MNYL9usKMGz" + \
            "hgp656onGkSbQHZR3ZHsSsTXWP3SV5ih4QTTFunwB6C0TMQVs" + \
            "EGw1P49hhFktb3md+RC4DFP7ZOzfkd9nne2B mycomment"
    print(key_validate("string", key1))
    print(key_validate("string", key2))
    print(key_parse(key1))
    print(key_parse("abcdedfg"))
    print(key_parse("ssh-rsa somestringhere")[2])
    """


if __name__ == "__main__":
    main()



"""managing information from GIT"""
from __future__ import print_function
from pprint import pprint
from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.util import banner


class GitInfo(object):

    """This class can be used to return some elementary information
    about the git hub directory.  This includes the:

    * version number of the code
    * the list of people contributing to the code
    * emails of the people contributing
    * a statistic about canged code by person

    Example::

        gitinfo = GitInfo()
        print gitinfo.authors()
        pprint(gitinfo.authors("dict"))
        pprint(gitinfo.emails())
        pprint(gitinfo.emails("dict"))
        pprint(gitinfo.info())
        print gitinfo.stat("laszewski@gmail.com")
        stats = gitinfo.compute()
        print stats

    """

    def __init__(self):
        """init method"""
        pass

    def version(self):
        '''
        retruns the verison of the code from github
        :rtype: the git tags
        '''
        return str(Shell.git("describe", "--tags"))[:-1]

    def emails(self, format_arg=None):
        '''
        returns the emails of the authors either as a text or as a dict. The
        format is specified as an argument.

        :param format_arg: if "dict" is specified a dict will be returned
        :rtype: dict or arry of e-mails dependent on format_arg
        '''
        if format_arg is None:
            format_string = "'%aN' <%cE>"
        elif format_arg == 'dict':
            format_string = "%aN\t%cE"
        #result = Shell.sort(Shell.git("log",
        #                              "--all",
        #                              "--format=" + format_string,
        #                              _tty_in=True,
        #                              _tty_out=False,
        #                             _piped=True), "-u")
        result = Shell.execute("git", ["log",
                                      "--all",
                                      "--format=" + format_string])


        if format_arg is None:
            return result
        elif format_arg == "dict":
            result = iter(result.replace("\n", "\t").split("\t")[:-1])
            emails = dict(zip(result, result))
            for name in emails:
                emails[name] = emails[name]
            return emails

    def authors(self, format_arg=None):
        '''
        returns the authors of the authors either as a text or as a dict. The
        format is specified as an argument.

        :param format_arg: if "dict" is specified a dict will be returned
        '''
        #result = Shell.git("shortlog", "-s", "-n", _tty_in=True, _tty_out=False)
        result = Shell.execute("git", ["shortlog", "-s", "-n"])

        if format_arg is None:
            return result
        elif format_arg == "dict":
            list_string = result.replace("\n", "\t").split("\t")[:-1]
            it = iter(list_string[::-1])
            authors = dict(zip(it, it))
            for name in authors:
                authors[name] = int(authors[name])
            return authors

    def info(self):
        '''
        returns the information about authors, emails, and commits in a dict.
        :rtype: a dict with the information
        '''
        authors = self.authors("dict")
        email = self.emails("dict")

        info = {}
        for name in authors:
            info[name] = {
                "name": name,
                "commits": authors[name],
                "email": email[name]}
        return info

    def stat(self, email):
        '''
        returns a statistic of a git author with the given e_mail.

        :param email: name of the author
        :rtype: a dict with the statistics
        '''
        sums = [0, 0, 0]
        for line in Shell.git("log", "--all", "--stat", '--author={0}'.format(email),
                              _tty_in=True,
                              _tty_out=False,
                              _iter=True):
            line = line[:-1]

            if " files changed" in line:
                line = line.replace(" insertions(+)", "")
                line = line.replace(" insertion(+)", "")
                line = line.replace(" deletion(-)", "")
                line = line.replace(" deletions(-)", "")
                line = line.replace(" files changed", "")
                line = line.split(",")
                data = [int(i) for i in line]
                for index in range(0, len(data)):
                    sums[index] += data[index]

        return {"email": email,
                "fileschanged": sums[0],
                "inserted": sums[1],
                "deleted": sums[2],
                "lineschanged": sums[1] + sums[2]}

    def compute(self):
        '''
        computes the statistic of all authors in a git repository and
        returns the result as a dict.
        :rtype: a dict with the values
        '''
        emails = set(gitinfo.emails("dict").values())

        stats = {}
        sums = {"fileschanged": 0, "inserted":
                0, "deleted": 0, "lineschanged": 0}
        for email in emails:
            print("Calculating stats for", email)
            stats[email] = gitinfo.stat(email)

            sums["fileschanged"] += stats[email]["fileschanged"]
            sums["inserted"] += stats[email]["inserted"]
            sums["deleted"] += stats[email]["deleted"]
            sums["lineschanged"] += stats[email]["lineschanged"]

        for email in emails:
            stats[email] = {'percentage': [
                stats[email]["fileschanged"] / float(sums["fileschanged"]),
                stats[email]["inserted"] / float(sums["inserted"]),
                stats[email]["deleted"] / float(sums["deleted"]),
                stats[email]["lineschanged"] / float(sums["lineschanged"])
            ]}

        return stats

def print_authors():
    gitinfo = GitInfo()
    print(gitinfo.authors())


if __name__ == "__main__":
    gitinfo = GitInfo()

    # print gitinfo.version()

    banner("a")
    print(gitinfo.authors())

    banner("b")
    pprint(gitinfo.authors("dict"))

    banner("c")
    pprint(gitinfo.emails())

    banner("d")
    pprint(gitinfo.emails("dict"))

    banner("e")
    pprint(gitinfo.info())

    banner("f")
    print(gitinfo.stat("laszewski@gmail.com"))

    banner("g")
    stats = gitinfo.compute()

    print(stats)

    banner("h")
    for email in stats:
        p = stats[email]["percentage"]
        print(("{0} {1:.3f}% {2:.3f}%  {3:.3f}% {4:.3f}%"
               .format(email, p[0], p[1], p[2], p[3])))



""" run with

python setup.py install; nosetests -v --nocapture  tests/cm_libcloud/test_libcloud_cli.py:Test_flavor.test_001

nosetests -v --nocapture tests/cm_libcloud/test_libcloud_cli.py

or

nosetests -v tests/cm_libcloud/test_libcloud_cli.py

"""

from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import HEADING
from cloudmesh_client.common.util import banner
from pprint import pprint

class Test_libcloud_cli:

    data = dotdict({
        "cloud": "aws",
    })

    def run(self, command):
        self.cloud = "aws"
        command = command.format(**self.data)
        banner(command, c="-")
        print(command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return result

    def setup(self):
        HEADING()
        result = self.run("cm default cloud=aws")
        assert "ok." in result

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_001(self):
        """
            Test VM refresh
        :return:
        """
        HEADING()
        result = self.run("cm vm refresh")
        assert "Refresh VMs for cloud aws" in result

    def test_002(self):
        """
            Test VM list
        :return:
        """
        HEADING()
        result = self.run("cm vm list")
        assert "Listing VMs on Cloud: aws" in result

    # def test_003(self):
    #     """
    #         Test Image refresh
    #     :return:
    #     """
    #     HEADING()
    #     result = self.run("cm image refresh")
    #     assert "Refresh image for cloud aws" in result

    def test_004(self):
        """
            Test Image list
        :return:
        """
        HEADING()
        result = self.run("cm image list")
        assert "name" in result

    def test_005(self):
        """
            Test Flavor refresh
        :return:
        """
        HEADING()
        result = self.run("cm flavor refresh")
        assert "Refresh flavor for cloud aws" in result

    def test_006(self):
        """
            Test Flavor list
        :return:
        """
        HEADING()
        result = self.run("cm flavor list")
        assert "RAM" in result

    def test_007(self):
        """
            Test VM instance boot
        :return:
        """
        HEADING()
        result = self.run("cm vm boot")
        assert "VM boot up success" in result

    def test_008(self):
        """
            Test VM login
        :return:
        """
        HEADING()
        result = self.run("cm vm ssh")
        assert "Refresh flavor for cloud aws" in result



from __future__ import print_function
from cloudmesh_client.shell.command import command, PluginCommand, CloudPluginCommand
from cloudmesh_client.shell.console import Console
from cloudmesh_client.default import Default


class RefreshCommand(PluginCommand, CloudPluginCommand):
    topics = {"refresh": "shell"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command refresh")
        #try:
        #    value = Default.get_refresh()
        #except:
        #    Default.set_refresh(True)

    # noinspection PyUnusedLocal
    @command
    def do_refresh(self, args, arguments):
        """
        ::

            Usage:
                refresh on
                refresh off
                refresh [list]

                switches on and off the refresh for clouds

        """
        if arguments["on"]:
            Default.set_refresh(True)
            Console.ok("Switch refresh on")
        elif arguments["off"]:
            Default.set_refresh(False)
            Console.ok("Switch refresh off")
        else:
            refresh = Default.refresh
            if refresh:
                msg = "on"
            else:
                msg = "off"
            Console.ok("Automatic cloud refresh is switched {}".format(msg))

        return ""



""" run with

nosetests -v --nocapture

or

nosetests -v

"""

from cloudmesh_client.common.util import HEADING


# noinspection PyMethodMayBeStatic,PyPep8Naming
class Test_pass:
    def setup(self):
        pass

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_dummy(self):
        HEADING()
        assert True



from pprint import pprint

class LibcloudDict(object):
    Libcloud_category_list = ['chameleon-ec2',
                              'cybera-ec2',
                              'aws']

    @staticmethod
    def convert_libcloud_vm_to_dict(node_obj):
        vm_dict = {
            'id': node_obj.id,
            'node_id': node_obj.id,
            'name': node_obj.name,
            'state': str(node_obj.state)
        }

        if len(node_obj.public_ips) > 0:
            vm_dict['public_ips'] = node_obj.public_ips[0]
        else:
            vm_dict['public_ips'] = ""
        if len(node_obj.private_ips) > 0:
            vm_dict['private_ips'] = node_obj.private_ips[0]
        else:
            vm_dict['private_ips'] = ""
        # TODO: Not sure why vm image details are required with 'node_obj.size'
        vm_image_dict = LibcloudDict.handle_vm_image_details(node_obj.size, is_image_dict=False)
        vm_dict.update(vm_image_dict)
        if node_obj.extra:
            extra_args_dict = LibcloudDict.handle_vm_extra_args(node_obj.extra)
            vm_dict.update(extra_args_dict)
            # pprint("Node details dict")
            # pprint(node_obj.extra)
        # pprint("IN convert_libcloud_vm_to_dict")
        # pprint(vm_dict)
        return vm_dict

    @staticmethod
    def handle_vm_extra_args(extra_args):
        extra_vm_dict = {}
        for key, value in list(extra_args.items()):
            if key == "availability":
                extra_vm_dict[key] = value
            if key == "instance_id":
                extra_vm_dict[key] = value
            if key == "instance_type":
                extra_vm_dict[key] = value
            if key == "key":
                extra_vm_dict[key] = value
            if key == "private_dns":
                extra_vm_dict[key] = value
            if key == "root_device_name":
                extra_vm_dict[key] = value
            if key == "root_device_type":
                extra_vm_dict[key] = value
            if key == "status":
                extra_vm_dict[key] = value
        return extra_vm_dict

    @staticmethod
    def handle_vm_size_details(node_size_obj):
        vm_size_dict = {}
        if node_size_obj.id:
            vm_size_dict['id'] = node_size_obj.id
            vm_size_dict['flavor_id'] = node_size_obj.id
        if node_size_obj.name:
            vm_size_dict['name'] = node_size_obj.name
        if node_size_obj.ram:
            vm_size_dict['ram'] = node_size_obj.ram
        if node_size_obj.name:
            vm_size_dict['disk'] = node_size_obj.disk
        if node_size_obj.name:
            vm_size_dict['bandwidth'] = node_size_obj.bandwidth
        if node_size_obj.name:
            vm_size_dict['price'] = node_size_obj.price
        if node_size_obj.extra:
            # TODO: Convert to Console
            pprint("Node Image size extra attrs")
            LibcloudDict.handle_vm_size_extra_args(node_size_obj.extra)
        return vm_size_dict

    @staticmethod
    def handle_vm_size_extra_args(node_size_extra_args):
        for key, val in list(node_size_extra_args.items()):
            # TODO: Convert to Console
            pprint(key + " : " + str(val))

    @staticmethod
    def handle_vm_image_details(node_image_obj, is_image_dict=True):
        node_image_dict = {}
        if is_image_dict:
            node_image_dict['id'] = node_image_obj.id
        if node_image_obj and node_image_obj.id:
            node_image_dict['image_id'] = node_image_obj.id
        else:
            node_image_dict['image_id'] = ""
        if node_image_obj and node_image_obj.name:
            node_image_dict["image_name"] = node_image_obj.name
        else:
            node_image_dict["image_name"] = ""
        if node_image_obj and node_image_obj.extra:
            vm_image_extra_args = LibcloudDict.handle_vm_image_extra_args(node_image_obj.extra)
            node_image_dict.update(vm_image_extra_args)
            # pprint("NodeImage extras")
            # pprint(node_image_obj.extra)
        else:
            # TODO: Convert to Console
            pprint("NodeImage extras not found")
        # Node Image Extra Args to be added here
        return node_image_dict

    @staticmethod
    def handle_vm_image_extra_args(extra_args):
        image_extra_args_dict = {}
        for key, value in list(extra_args.items()):
            if key == "architecture":
                image_extra_args_dict[key] = value
            if key == "description":
                image_extra_args_dict[key] = value
            if key == "hypervisor":
                image_extra_args_dict[key] = value
            if key == "image_location":
                image_extra_args_dict[key] = value
            if key == "image_type":
                image_extra_args_dict[key] = value
            if key == "is_public":
                image_extra_args_dict[key] = value
            if key == "kernel_id":
                image_extra_args_dict[key] = value
            if key == "owner_alias":
                image_extra_args_dict[key] = value
            if key == "owner_id":
                image_extra_args_dict[key] = value
            if key == "platform":
                image_extra_args_dict[key] = value
            if key == "ramdisk_id":
                image_extra_args_dict[key] = value
            if key == "state":
                image_extra_args_dict[key] = value
            if key == "virtualization_type":
                image_extra_args_dict[key] = value
        return image_extra_args_dict

    @staticmethod
    def handle_key_details(key_obj):
        # Simply converts all attributes to dict
        key_dict = key_obj.__dict__
        # Selects attributes to convert
        key_dict = {
                'name': key_obj.name,
                'fingerprint': key_obj.fingerprint,
                'driver': key_obj.driver
                }
        return key_dict



from cloudmesh_client.cloud.counter import Counter
from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.ConfigDict import ConfigDict


# noinspection PyUnusedLocal,PyBroadException
class BatchProviderBase(object):
    prefix = "job"
    jobid = 0

    @classmethod
    def queue(cls, **kwargs):
        ValueError("Not yet implemented")

    @classmethod
    def delete(cls, **kwargs):
        ValueError("Not yet implemented")

    @classmethod
    def run(cls, **kwargs):
        ValueError("Not yet implemented")

    @classmethod
    def counter(cls):
        cls.jobid = Counter.get()
        return cls.jobid

    @classmethod
    def incr(cls):
        Counter.incr()
        cls.jobid = Counter.get()
        return cls.jobid

    @classmethod
    def jobname(cls, counter=None):
        if counter is None:
            counter = cls.counter()
        data = {
            "counter": counter,
            "prefix": cls.prefix
        }
        return "{prefix}-{counter}".format(**data)

    @classmethod
    def create_remote_dir(cls, cluster, directory):
        Shell.ssh(cluster, "mkdir -p {dir}".format(dir=directory))

    @classmethod
    def read_config(cls, cluster):
        """
        reads in the cluster config from the yaml file and returns the specific cluster information

        newhpc:
            experiment:
                    name: gregor-00000
            active:
            - comet
            - juliet
            clusters:
                india:
                    cm_heading: India HPC CLuster
                    cm_host: india
                    cm_label: indiahpc
                    cm_type: slurm
                    cm_type_version: 14.11.9
                    credentials:
                        username: gregor
                        project: None
                    default:
                        queue: delta
                        experiment_dir: ./experiment

        :param cls:
        :param cluster:
        :return:
        """

        try:
            config = cls.config
        except:
            cls.config = None
        if cls.config is None:
            cls.config = ConfigDict("cloudmesh.yaml")["cloudmesh.hpc"]
            cls.experiment_name_format = cls.config["experiment"]["name"]
        return cls.config["clusters"][cluster]



from __future__ import print_function

from cloudmesh_client.shell.console import Console
from cloudmesh_client.shell.command import command
from cloudmesh_client.common.menu import menu_return_num
from cloudmesh_client.default import Default
from cloudmesh_client.cloud.image import Image
from cloudmesh_client.cloud.flavor import Flavor
from cloudmesh_client.common.ConfigDict import ConfigDict
from cloudmesh_client.shell.command import PluginCommand, CloudPluginCommand
from cloudmesh_client.cloud.key import Key

# noinspection PyBroadException,PyBroadException
class SelectCommand(PluginCommand, CloudPluginCommand):
    topics = {"select": "cloud"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command select")

    # noinspection PyUnusedLocal
    @command
    def do_select(self, args, arguments):
        """
        ::

          Usage:
              select image [CLOUD] [--refresh]
              select flavor [CLOUD] [--refresh]
              select cloud [CLOUD]
              select key [CLOUD]

          selects interactively the default values

          Arguments:

            CLOUD    the name of the cloud

          Options:

            --refresh   refreshes the data before displaying it
                        from the cloud

        """
        # pprint(arguments)
        cloud = arguments["CLOUD"] or Default.cloud
        if arguments["image"]:
            try:
                refresh = arguments['--refresh'] or Default.refresh
                if refresh:
                    Image.refresh(cloud)

                image_dict = Image.list(cloud, format="dict")

                image_names = list()
                for image in list(image_dict.values()):
                    image_names.append(image["name"])

                number = menu_return_num(title="Select an Image",
                                         menu_list=image_names,
                                         tries=10,
                                         with_display=True)

                if number == "q":
                    pass
                else:
                    image = image_names[number]
                    print("Selected image " + image)
                    Default.set("image", image, category=cloud)
            except:
                print("ERROR: could not set image.")

        elif arguments["flavor"]:
            try:
                refresh = arguments['--refresh'] or Default.refresh
                if refresh:
                    Flavor.refresh(cloud)

                flavor_dict = Flavor.list(cloud, format="dict")

                flavor_names = list()
                for flavor in list(flavor_dict.values()):
                    flavor_names.append(flavor["name"])

                number = menu_return_num(title="Select a Flavor",
                                         menu_list=flavor_names,
                                         tries=10,
                                         with_display=True)

                if number == "q":
                    pass
                else:
                    flavor = flavor_names[number]
                    print("Selected flavor " + flavor)
                    Default.set("flavor", flavor, category=cloud)
            except:
                print("ERROR: could not set flavor.")

        elif arguments["cloud"]:
            try:
                config = ConfigDict("cloudmesh.yaml")
                clouds = config["cloudmesh"]["clouds"]

                for key in clouds:
                    Console.ok("  " + key)

                number = menu_return_num(title="Select a cloud",
                                         menu_list=list(clouds),
                                         tries=10,
                                         with_display=True)
                if number == "q":
                    pass
                else:
                    cloud = list(clouds)[number]
                    print("Selected cloud " + cloud)
                    Default.set("cloud", cloud, "general")
            except:
                print("ERROR: could not set cloud.")

        elif arguments["key"]:
            try:
                #db = SSHKeyDBManager()

                key_dict = Key.all(output='dict')

                key_names = list()
                for key in key_dict.values():
                    key_names.append(key["name"])

                number = menu_return_num(title="Select a Key",
                                         menu_list=key_names,
                                         tries=10,
                                         with_display=True)

                if number == "q":
                    pass
                else:
                    key = key_names[number]
                    print("Selected key " + key)

                    # TODO Fix default key setting in key DB
                    # db.set_default(key)

                    Default.set("key", key, category=cloud)
            except:
                print("ERROR: could not set key")

        return ""




""" run with

python setup.py install; nosetests -v --nocapture tests/cm_cloud/test_nova.py:Test_nova.test_001

nosetests -v --nocapture tests/test_nova.py

or

nosetests -v tests/test_nova.py

"""

from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import HEADING
from cloudmesh_client.common.util import banner
from cloudmesh_client.default import Default


# noinspection PyPep8Naming
class Test_nova:
    """tests nova command"""

    data = dotdict({
        "cloud": Default.cloud,
        "group": "mygroup",
        "wrong_cloud": "no_cloud"
    })

    def run(self, command):
        command = command.format(**self.data)
        banner(command, c="-")
        print(command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return str(result)

    def setup(self):
        pass

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_000(self):
        HEADING("set the default group")
        result = self.run("cm default group={group}")
        # assert "{cloud} is set".format(**self.data) in result

    def test_001(self):
        HEADING("cm nova set <cloud>")
        result = self.run("cm nova set {cloud}")
        assert "{cloud} is set".format(**self.data) in result

    def test_002(self):
        HEADING("cm nova info <cloud>")
        result = self.run("cm nova info {cloud}")
        assert "OK." in result

    def test_003(self):
        HEADING("cm nova list")
        result = self.run("cm nova list")
        assert "+" in result

    def test_004(self):
        HEADING("cm nova image-list")
        result = self.run("cm nova image-list")
        assert "ACTIVE" in result



from __future__ import print_function
from string import Template
import inspect
import glob
import os
import shutil
import collections
# import pip
import time
import sys
import re

from builtins import input
from past.builtins import basestring
import random


def exponential_backoff(fn, sleeptime_s_max=30*60):
    """Calls `fn` until it returns True, with an exponentially increasing wait time between calls"""

    sleeptime_ms = 500
    while True:
        if fn() == True:
            return True
        else:
            print ('Sleepting {} ms'.format(sleeptime_ms))
            time.sleep(sleeptime_ms / 1000.0)
            sleeptime_ms *= 2

        if sleeptime_ms/1000.0 > sleeptime_s_max:
            return False


def search(lines, pattern):
    p = pattern.replace("*", ".*")
    test = re.compile(p)
    result = []
    for l in lines:
        if test.search(l):
            result.append(l)
    return result

def grep(pattern, filename):
    """Very simple grep that returns the first matching line in a file.
    String matching only, does not do REs as currently implemented.
    """
    try:
        # for line in file
        # if line matches pattern:
        #    return line
        return next((L for L in open(filename) if L.find(pattern) >= 0))
    except StopIteration:
        return ''


def path_expand(text):
    """ returns a string with expanded variable.

    :param text: the path to be expanded, which can include ~ and $ variables
    :param text: string

    """
    template = Template(text)
    result = template.substitute(os.environ)
    result = os.path.expanduser(result)
    return result


def convert_from_unicode(data):
    if isinstance(data, basestring):
        return str(data)
    elif isinstance(data, collections.Mapping):
        return dict(map(convert_from_unicode, data.items()))
    elif isinstance(data, collections.Iterable):
        return type(data)(map(convert_from_unicode, data))
    else:
        return data


def yn_choice(message, default='y', tries=None):
    """asks for a yes/no question.
    :param message: the message containing the question
    :param default: the default answer
    """
    # http://stackoverflow.com/questions/3041986/python-command-line-yes-no-input"""
    choices = 'Y/n' if default.lower() in ('y', 'yes') else 'y/N'
    if tries is None:
        choice = input("%s (%s) " % (message, choices))
        values = ('y', 'yes', '') if default == 'y' else ('y', 'yes')
        return True if choice.strip().lower() in values else False
    else:
        while tries > 0:
            choice = input("%s (%s) (%s)" % (message, choices, "'q' to discard"))
            choice = choice.strip().lower()
            if choice in ['y', 'yes']:
                return True
            elif choice in ['n', 'no', 'q']:
                return False
            else:
                print("Invalid input...")
                tries -= 1


def banner(txt=None, c="#", debug=True):
    """prints a banner of the form with a frame of # arround the txt::

      ############################
      # txt
      ############################

    .

    :param txt: a text message to be printed
    :type txt: string
    :param c: the character used instead of c
    :type c: character
    """
    if debug:
        print
        print("#", 70 * c)
        if txt is not None:
            print("#", txt)
            print("#", 70 * c)


def str_banner(txt=None, c="#", debug=True):
    """prints a banner of the form with a frame of # arround the txt::

      ############################
      # txt
      ############################

    .

    :param txt: a text message to be printed
    :type txt: string
    :param c: the character used instead of c
    :type c: character
    """
    str = ""
    if debug:
        str += "\n"
        str += "# " + 70 * c
        if txt is not None:
            str += "# " + txt
            str += "# " + 70 * c
    return str


# noinspection PyPep8Naming
def HEADING(txt=None):
    """
    Prints a message to stdout with #### surrounding it. This is useful for
    nosetests to better distinguish them.

    :param txt: a text message to be printed
    :type txt: string
    """
    frame = inspect.getouterframes(inspect.currentframe())

    filename = frame[1][1].replace(os.getcwd(), "")
    line = frame[1][2] - 1
    method = frame[1][3]
    msg = "{}\n# {} {} {}".format(txt, method, filename, line)

    print()
    banner(msg)


def backup_name(filename):
    """
    :param filename: given a filename creates a backup name of the form
                     filename.bak.1. If the filename already exists
                     the number will be increased as  much as needed so
                     the file does not exist in the given location.
                     The filename can consists a path and is expanded
                     with ~ and environment variables.
    :type filename: string
    :rtype: string
    """
    location = path_expand(filename)
    n = 0
    found = True
    backup = None
    while found:
        n += 1
        backup = "{0}.bak.{1}".format(location, n)
        found = os.path.isfile(backup)
    return backup


def auto_create_version(class_name, version, filename="__init__.py"):
    version_filename = "{0}/{1}".format(class_name, filename)
    with open(version_filename, "r") as f:
        content = f.read()

    if content != '__version__ = "{0}"'.format(version):
        banner("Updating version to {0}".format(version))
        with open(version_filename, "w") as text_file:
            text_file.write('__version__ = "{0:s}"'.format(version))


def auto_create_requirements(requirements):
    """
    creates a requirement.txt file form the requirements in the list. If the file exists, it get changed only if the
    requirements in the list are different from the existing file

    :param requirements: the requirements in a list
    """
    banner("Creating requirements.txt file")
    try:
        with open("requirements.txt", "r") as f:
            file_content = f.read()
    except:
        file_content = ""

    setup_requirements = '\n'.join(requirements)

    if setup_requirements != file_content:
        with open("requirements.txt", "w") as text_file:
            text_file.write(setup_requirements)


def copy_files(files_glob, source_dir, dest_dir):
    """

    :param files_glob: *.yaml
    :param source_dir: source directiry
    :param dest_dir: destination directory
    :return:
    """
    files = glob.iglob(os.path.join(source_dir, files_glob))
    for filename in files:
        if os.path.isfile(filename):
            shutil.copy2(filename, dest_dir)


def dict_replace(content, replacements=None):
    if replacements is None:
        replacements = {}
    for key in replacements:
        content = content.replace("\{key\}".format(replacements[key]))

    return content


def readfile(filename):
    with open(path_expand(filename), 'r') as f:
        content = f.read()
    return content


def writefile(filename, content):
    outfile = open(path_expand(filename), 'w')
    outfile.write(content)
    outfile.close()


def get_python():
    python_version = sys.version_info[:3]
    v_string = [str(i) for i in python_version]
    python_version_s = '.'.join(v_string)

    # pip_version = pip.__version__
    pip_version = "8.1.2"
    return python_version_s, pip_version


def check_python():
    python_version = sys.version_info[:3]

    v_string = [str(i) for i in python_version]

    python_version_s = '.'.join(v_string)
    if (python_version[0] == 2) and (python_version[1] >= 7) and (python_version[2] >= 9):

        print("You are running a supported version of python: {:}".format(python_version_s))
    else:
        print("WARNING: You are running an unsupported version of python: {:}".format(python_version_s))
        print("         We recommend you update your python")

    # pip_version = pip.__version__
    pip_version = "8.1.2"

    if int(pip_version.split(".")[0]) >= 7:
        print("You are running a supported version of pip: " + str(pip_version))
    else:
        print("WARNING: You are running an old version of pip: " + str(pip_version))
        print("         We recommend you update your pip  with \n")
        print("             pip install -U pip\n")

# Reference: http://interactivepython.org/runestone/static/everyday/2013/01/3_password.html
def generate_password(length=8, lower=True, upper=True, number=True):
    lletters = "abcdefghijklmnopqrstuvwxyz"
    uletters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    # This doesn't guarantee both lower and upper cases will show up
    alphabet = lletters + uletters
    digit = "0123456789"
    mypw = ""

    def _random_character(texts):
        return texts[random.randrange(len(texts))]

    if not lower:
        alphabet = uletters
    elif not upper:
        alphabet = lletters

    for i in range(length):
        # last half length will be filled with numbers
        if number and i >= (length / 2):
            mypw = mypw + _random_character(digit)
        else:
            mypw = mypw + _random_character(alphabet)
    return mypw



class Attributes(object):
    @classmethod
    def get(cls, kind):
        layout = {
            'flavor': {
                'order': [
                    'id',
                    'name',
                    'user',
                    'cpu',
                    'ram',
                    'bandwidth',
                    'price',
                    'category',
                    'uuid',
                    'updated_at'
                ],
                'header': [
                    'Id',
                    'Name',
                    'User',
                    'cpu',
                    'RAM',
                    'bandwidth',
                    'price',
                    'Cloud',
                    'UUID',
                    'Updated'
                ]
            },
            'image': {
                'order': [
                    'id',
                    'name',
                    'category',
                    'image_type',
                    'state',
                    'uuid',
                    'updated_at',
                    'owner_id'
                ],
                'header': [
                    'id',
                    'name',
                    'cloud',
                    'image_type',
                    'state',
                    'uuid',
                    'updated_at',
                    'owner_id'
                ]
            },
            'vm': {
                'order': [
                    'id',
                    'uuid',
                    'label',
                    'status',
                    'public_ips',
                    'private_ips',
                    'image_name',
                    'key',
                    'availability',
                    'instance_type',
                    'user',
                    'category',
                    'updated_at'
                ],
                'header': [
                    'id',
                    'uuid',
                    'label',
                    'status',
                    'public_ips',
                    'private_ips',
                    'image_name',
                    'key',
                    'availability',
                    'instance_type',
                    'user',
                    'cloud',
                    'updated'
                ]
            },
            'floating_ip': {
                'order': [
                    "instance_name",
                    "ip",
                    "pool",
                    "fixed_ip",
                    "id",
                    "instance_id",
                    'cloud',
                    'updated'
                ],
                'header': [
                    "instance_name",
                    "floating_ip",
                    "floating_ip_pool",
                    "fixed_ip",
                    "floating_ip_id",
                    "instance_id",
                    'cloud',
                    'updated'
                ],
            },
            'floating_ip_pool': {
                'order': [
                    "name"
                ],
                'header': [
                    "floating_ip_pool"
                ],
            },
            'clouds': {
                'order': [
                    "cloud",
                    "status"
                ],
                'header': [
                    "cloud",
                    "status"
                ],
            },
            'limits': {
                'order': [
                    'Name',
                    'Value'
                ],
                'header': [
                    'Name',
                    'Value'
                ]
            },
            'quota': {
                'order': [
                    'Quota',
                    'Limit'
                ],
                'header': [
                    'Quota',
                    'Limit'
                ]
            },
            'secgroup': {
                'order': [
                    'id',
                    'name',
                    'category',
                    'user',
                    'project',
                    'uuid'
                ],
                'header': [
                    'id',
                    'secgroup_name',
                    'category',
                    'user',
                    'tenant_id',
                    'secgroup_uuid'
                ]
            },
            'default': {
                'order': [
                    'user',
                    'cloud',
                    'name',
                    'value',
                    'created_at',
                    'updated_at'
                ],
                'header': [
                    'user',
                    'cloud',
                    'name',
                    'value',
                    'created',
                    'updated'
                ],
            }
        }
        if kind in layout:
            order = layout[kind]['order']
            header = layout[kind]['header']
        else:
            order = None
            header = None

        return order, header



import requests
from cloudmesh_client.cloud.ListResource import ListResource
from cloudmesh_client.cloud.iaas.CloudProvider import CloudProvider
from cloudmesh_client.common.Printer import Printer

requests.packages.urllib3.disable_warnings()


class Limits(ListResource):

    @classmethod
    def list(cls, cloud, output="table", tenant=None):
        try:
            provider = CloudProvider(cloud).provider
            # if tenant is None:
            #     tenant = provider.tenant

            result = provider.list_limits(tenant)["absolute"]

            (order, header) = CloudProvider(cloud).get_attributes("limits")

            return Printer.attribute(result,
                                     header=header,
                                     output=output)
        except Exception as e:
            return e



""" run with

python setup.py install; nosetests -v --nocapture  tests/cm_cloud/test_secgroup.py:Test_secgroup.test_001

nosetests -v --nocapture tests/test_secgroup.py

or

nosetests -v tests/test_secgroup.py

"""

from cloudmesh_client.common.ConfigDict import ConfigDict
from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import HEADING
from cloudmesh_client.common.util import banner
from cloudmesh_client.default import Default

from cloudmesh_client.cloud.iaas.CloudProvider import CloudProvider
from pprint import pprint
from cloudmesh_client.shell.console import Console
from cloudmesh_client.db.CloudmeshDatabase import CloudmeshDatabase
'''
cm secgroup list
cm secgroup list --cloud=kilo
cm secgroup add  cm-gregor-default web 80 80 tcp  0.0.0.0/0
cm secgroup add  cm-gregor-default ssh 22 22 tcp  0.0.0.0/0
cm secgroup upload --cloud=kilo
'''


# noinspection PyPep8Naming
class Test_secgroup:

    data = dotdict({
        "dgroup": "{}-default".format(Default.user),
        "cloud": Default.cloud,
        "group": "cm-{}-test".format(Default.user),
        "wrong_cloud": "no_cloud",
        "rules": {
            "rule_http": "80 80 tcp  0.0.0.0/0",
            "rule_https": "443 443 tcp  0.0.0.0/0",
            "rule_ssh": "22 22 tcp  0.0.0.0/0",
        },
        "image": Default.get_image(category=Default.cloud),
        "flavor": Default.get_flavor(category=Default.cloud),
        "vm": "{}_testsecgroup".format(Default.user),
    })
    for rule in data.rules:
        data[rule] = data.rules[rule]

    data.tenant = ConfigDict("cloudmesh.yaml")["cloudmesh.clouds"][data.cloud]["credentials"]["OS_TENANT_NAME"]

    def run(self, command):
        command = command.format(**self.data)
        banner(command, c="-")
        print(command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return str(result)

    def setup(self):
        pass

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_001(self):
        HEADING("cm secgroup add ...")
        for rule in self.data.rules:
            command = "cm secgroup add  {} {} {}".format(self.data.group, rule, self.data["rules"][rule])
            result = self.run(command)

        command = "cm secgroup list"
        result = self.run(command)

        for rule in self.data.rules:
            assert rule in result

    def test_002(self):
        HEADING("cm secgroup delete ...")

        command = "cm secgroup delete {group}".format(**self.data)

        result = self.run(command)

        result = self.run("cm secgroup list")

        assert self.data.group not in result

    def test_003(self):

        HEADING("secgroup api")

        data = dotdict({
            'cloud': "cm",
            'groupname': 'gvonlasz-default'
        })
        provider = CloudProvider(data.cloud).provider

        groups = None
        rules = provider.list_secgroup(data.cloud)

        """
        # passed
        def add_group(cloud, groupname):
            provider = CloudProvider(cloud).provider
            return provider.create_secgroup(groupname)

        # passed
        def delete_group(cloud, groupname):
            provider = CloudProvider(data.cloud).provider
            return provider.delete_secgroup(groupname)

        # passed
        def add_rule(cloud, groupname, rulename):
            ret = None
            provider = CloudProvider(data.cloud).provider
            # fetch rule from db
            cm = CloudmeshDatabase()
            db_rule = cm.find(kind="secgrouprule", category="general", name=rulename, scope='first', output='dict')
            kwargs = {}
            kwargs["protocol"] = db_rule["protocol"]
            kwargs["cidr"] = db_rule["cidr"]
            kwargs["from_port"] = db_rule["fromPort"]
            kwargs["to_port"] = db_rule["toPort"]
            group = get_group(cloud, groupname)
            if group:
                groupid = group["id"]
                kwargs["uuid"] = groupid
                print (kwargs)
                ret = provider.add_secgroup_rule(**kwargs)
            return ret

        # passed
        def delete_rule(cloud, groupname, rulename):
            ret = None
            provider = CloudProvider(data.cloud).provider
            ruleid = get_rule(cloud, groupname, rulename)
            if ruleid:
                ret = provider.delete_secgroup_rule(ruleid)
            else:
                Console.error("Rule does not exist - Rule:{}, Group:{}".format(rulename, groupname))
            return ret

        # passed
        def list_groups(cloud):
            provider = CloudProvider(cloud).provider
            groups = provider.list_secgroup(cloud)
            return groups

        # passed
        def get_group(cloud, groupname):
            provider = CloudProvider(cloud).provider
            groups = provider.list_secgroup(cloud)
            ret = None
            for groupkey in groups:
                group = groups[groupkey]
                if group["name"] == groupname:
                    ret = group
                    break
            return ret

        # passed
        def list_rules(cloud, groupname):
            provider = CloudProvider(cloud).provider
            groups = provider.list_secgroup(cloud)
            for id in groups:
                group = groups[id]
                if groupname == group["name"]:
                    return group["rules"]
            return None

        # passed
        def get_rule(cloud, groupname, rulename):
            rules = list_rules(cloud, groupname)
            # find properties for db rule

            cm = CloudmeshDatabase()
            db_rule = cm.find(kind="secgrouprule", category="general", name=rulename, scope='first', output='dict')

            #db_rule = {u'from_port': 80,
            #          u'ip_protocol': u'tcp',
            #          u'ip_range': {u'cidr': u'0.0.0.0/0'},
            #          u'to_port': 80},
            ruleid = None
            for rule in rules:

                if 'cidr' in rule['ip_range']:
                    if (db_rule["fromPort"] == str(rule['from_port']) and
                        db_rule["toPort"] == str(rule['to_port']) and
                        db_rule["protocol"] == rule['ip_protocol'] and
                        db_rule["cidr"] == rule['ip_range']['cidr']
                        ):
                        ruleid = rule['id'] #uuid for the rule
            return ruleid
        """

        # testing each individual method defined in this test
        #
        # list/get of groups/rules
        # all passed
        # pprint (list_groups(data.cloud))
        # print("dgroup", self.data.dgroup)
        # dgroup = self.data.dgroup
        # pprint (get_group(data.cloud, "default"))
        # pprint (list_rules(data.cloud, "default"))
        # pprint (get_rule(data.cloud, "default", "ssh"))

        # print ("...should be None")
        # pprint(get_group(data.cloud, dgroup))

        '''
        # testing adding and deleting groups
        # all passed
        print ("...should be None")
        pprint(get_group(data.cloud, "fwtest"))
        print ("...should return the newly created object")
        pprint(add_group(data.cloud, "fwtest"))
        print ("...should be NOT None")
        pprint(get_group(data.cloud, "fwtest"))
        print ("...deleting")
        pprint(delete_group(data.cloud, "fwtest"))
        print ("...should be None")
        pprint(get_group(data.cloud, "fwtest"))
        '''

        '''
        # testing adding and deleting rules
        # all passed
        print ("...should return the newly created object")
        pprint(add_group(data.cloud, "fwtest"))
        print ("...adding ssh rule")
        pprint(add_rule(data.cloud, "fwtest", "ssh"))
        print ("...listing the group. Should have the ssh rule int it")
        pprint(get_group(data.cloud, "fwtest"))
        print ("...deleting ssh rule")
        pprint(delete_rule(data.cloud, "fwtest", "ssh"))
        print ("...listing the group. ssh rule should be gone")
        pprint(get_group(data.cloud, "fwtest"))
        print ("...deleting the testing group")
        pprint(delete_group(data.cloud, "fwtest"))
        '''

    def test_004(self):
        '''
        A through test of deleting/updating secgroups.
        It creates in the db a testing group with 3 rules;
        Uploading the secgroup to cloud;
        booting a vm with this newly created group;
        deleting the secgroup (should fail as being used);
        updating rules for the secgroup
        (should succeed - updating rules, but not creating duplicated group);
        cleaning up...
            delete the testing vm
            deleting the testing secgroup
        '''

        HEADING("creating testing group in db and populate with rules")
        for rule in self.data.rules:
            command = "cm secgroup add  {} {} {}".format(self.data.group, rule, self.data["rules"][rule])
            result = self.run(command)

        HEADING("uploading the newly created secgroup to default cloud")
        command = "cm secgroup upload {} --cloud={}".format(self.data.group, self.data.cloud)
        result = self.run(command)

        HEADING("listing secgroup in default cloud")
        command = "cm secgroup list --cloud={}".format(self.data.cloud)
        result = self.run(command)
        assert "80" in result and self.data.group in result

        HEADING("booting a vm with the test secgroup")
        command = "cm vm boot --name={vm} --cloud={cloud} --image={image}" + \
                  " --flavor={flavor} --secgroup={group}"
        result = self.run(command)
        assert "OK." in result

        HEADING("cm secgroup delete ... Deleting a secgroup that is being used")
        command = "cm secgroup delete {} --cloud={}".format(self.data.group, self.data.cloud)
        result = self.run(command)
        assert "ERROR" in result and "in use" in result

        HEADING("adding new rule to the test secgroup (change in db)")
        command = "cm secgroup add {} ssh 8765 8765 tcp 0.0.0.0/0".format(self.data.group)
        result = self.run(command)
        HEADING("updating the teseting secgroup in the default cloud (upload to cloud)")
        command = "cm secgroup upload {} --cloud={}".format(self.data.group, self.data.cloud)
        result = self.run(command)

        HEADING("cm secgroup list --cloud={}".format(self.data.cloud))
        command = "cm secgroup list --cloud={}".format(self.data.cloud)
        result = self.run(command)
        assert "8765" in result

        HEADING("Cleaning up......")
        HEADING("deleting the test vm and the test secgroup")
        command = "cm vm delete {}".format(self.data.vm)
        result = self.run(command)
        command = "cm secgroup delete {} --cloud={}".format(self.data.group, self.data.cloud)
        result = self.run(command)
        command = "cm secgroup list --cloud={}".format(self.data.cloud)
        result = self.run(command)
        assert self.data.group not in result



import json
from cloudmesh_client.cloud.hpc.BatchProvider import BatchProvider


# TODO: is this outdated and superseded by hpc.py?
class Squeue(object):
    @staticmethod
    def squeue_to_json(input_str):
        d = {}
        for i, line in enumerate(input_str.splitlines()):
            if not line.startswith('Warning:') and not line.__contains__(
                    'NODELIST(REASON)'):
                d[i] = {}
                # noinspection PyPep8
                d[i]['jobid'], \
                d[i]['partition'], \
                d[i]['name'], \
                d[i]['user'], \
                d[i]['st'], \
                d[i]['time'], \
                d[i]['nodes'], \
                d[i]['nodelist(reason)'] = line.split()

        json_str = json.dumps(d, indent=4, separators=(',', ': '))
        # assert returned value is string
        assert isinstance(json_str, str)
        # convert to json obj
        return json.loads(json_str)

    @staticmethod
    def get(user=None, name=None):
        squeue_json = Squeue.read_squeue()
        # assert returned value is string
        assert isinstance(squeue_json, str)
        # convert to json obj
        json_obj = json.loads(squeue_json)

        d = {}
        for i, key in enumerate(json_obj.keys()):
            # both user, name parameters are specified
            if user and name:
                if user == json_obj[key]["user"] and name == json_obj[key]["name"]:
                    d[i] = json_obj[key]
            # only user parameter is specified
            elif user:
                if user == json_obj[key]["user"]:
                    d[i] = json_obj[key]
            # only name parameter is specified
            elif name:
                if name == json_obj[key]["name"]:
                    d[i] = json_obj[key]
            # no parameters specified, return all
            else:
                d[i] = json_obj[key]

        # return json dump
        return json.dumps(d, indent=4, separators=(',', ': '))

    @staticmethod
    def read_squeue():
        # read squeue from comet
        # TODO: check this function
        name = None
        provider = BatchProvider(name)
        return provider.read_squeue(format="json")


if __name__ == "__main__":
    # TODO: user should be read from yaml file
    print(Squeue.read_squeue())
    print(Squeue.get(name="NGBW-JOB", user="cipres"))
    # print(Squeue.get(name="NGBW-JOB"))
    # print(Squeue.get(user="cipres"))
    # print(Squeue.get())



from __future__ import print_function

import os
from pprint import pprint

from cloudmesh_client.common.ConfigDict import path_expand
from cloudmesh_client.common.Printer import Printer
from cloudmesh_client.common.ssh_config import ssh_config
from cloudmesh_client.common.todo import TODO
from cloudmesh_client.common.util import banner
from cloudmesh_client.shell.command import PluginCommand, ShellPluginCommand, \
    CometPluginCommand
from cloudmesh_client.shell.command import command
from cloudmesh_client.shell.console import Console


class SecureShellCommand(PluginCommand, ShellPluginCommand, CometPluginCommand):
    # def activate_cm_shell_ssh(self):

    topics = {"ssh": "security"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command ssh")

    # noinspection PyUnusedLocal
    @command
    def do_ssh(self, args, arguments):
        """
        ::

            Usage:
                ssh table
                ssh list [--format=FORMAT]
                ssh cat
                ssh register NAME PARAMETERS
                ssh ARGUMENTS


            conducts a ssh login on a machine while using a set of
            registered machines specified in ~/.ssh/config

            Arguments:

              NAME        Name or ip of the machine to log in
              list        Lists the machines that are registered and
                          the commands to login to them
              PARAMETERS  Register te resource and add the given
                          parameters to the ssh config file.  if the
                          resoource exists, it will be overwritten. The
                          information will be written in /.ssh/config

            Options:

               -v       verbose mode
               --format=FORMAT   the format in which this list is given
                                 formats incluse table, json, yaml, dict
                                 [default: table]

               --user=USER       overwrites the username that is
                                 specified in ~/.ssh/config

               --key=KEY         The keyname as defined in the key list
                                 or a location that contains a pblic key

            Description:

                ssh list
                    lists the hostsnames  that are present in the
                    ~/.ssh/config file

                ssh cat
                    prints the ~/.ssh/config file

                ssh table
                    prints contents of the ~/.ssh/config file in table format

                ssh register NAME PARAMETERS
                    registers a host i ~/.ssh/config file
                    Parameters are attribute=value pairs
                    Note: Note yet implemented

                ssh ARGUMENTS
                    executes the ssh command with the given arguments
                    Example:
                        ssh myhost

                            conducts an ssh login to myhost if it is defined in
                            ~/.ssh/config file
        """
        # pprint(arguments)

        def read(filename=None):
            if filename is None:
                filename = "~/.ssh/config"
            with open(path_expand("~/.ssh/config"), "r") as f:
                content = f.readlines()
            return "".join(content)

        def print_ssh_table(output):
            content = read(filename="~/.ssh/config").split("\n")

            entries = [
            ]

            def empty():
                return {
                    "host": None,
                    "hostname": None,
                    "user": None,
                    "proxycommand": None,
                    "serveraliveinterval": None,
                    "localforward": None,
                    "forwardx11": None
                }

            entry = empty()
            for line in content:
                line = line.strip()
                if line.startswith("#"):
                    pass
                elif line.strip() == "":
                    pass
                elif "Host " in line:
                    hostname = line.strip().split("Host")[1]
                    entry["host"] = hostname.strip()
                    if entry is not None:
                        entries.append(entry)
                    entry = empty()
                else:
                    attribute, value = line.strip().split(" ", 1)
                    entry[attribute.lower()] = value.strip()

            # pprint(entries)
            order = ["host",
                     "hostname",
                     "user",
                     "proxycommand",
                     "serveraliveinterval",
                     "localforward",
                     "forwardx11"]

            print(Printer.list(entries, order=order))

        if arguments["list"]:

            output_format = arguments["--format"]

            if output_format is None:
                banner('List SSH config hosts')
                hosts = ssh_config()
                for host in hosts.list():
                    print(host)
            else:

                print_ssh_table(output_format)

            return ""

        elif arguments["table"]:

            print_ssh_table()
            return ""

        elif arguments["cat"]:

            print(read(filename="~/.ssh/config"))

        elif arguments["register"]:

            name = arguments["NAME"]
            parameters = arguments["PARAMETERS"]
            Console.ok('register {} {}'.format(name, parameters))
            TODO.implement("Not implemented")

        else:  # ssh ARGUMENTS...

            args = arguments["ARGUMENTS"]
            os.system("ssh {}".format(args))
            return ""

        return ""




""" run with

python setup.py install; nosetests -v --nocapture  tests/cm_cloud/test_OS_API.py:Test_osapi.test_001
python setup.py install; py.test tests/cm_cloud/test_OS_API.py:Test_osapi.test_001
nosetests -v --nocapture tests/test_list.py

or

nosetests -v tests/test_list.py

"""

from pprint import pprint

from cloudmesh_client import ConfigDict
from cloudmesh_client.cloud.iaas.provider.openstack.CloudProviderOpenstackAPI import CloudProviderOpenstackAPI
from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import HEADING
from cloudmesh_client.common.util import banner
from cloudmesh_client.default import Default


# noinspection PyPep8Naming
class Test_osapi:
    data = dotdict({
        "cloud": Default.cloud,
        "format": "json",
        "user": "fake",
        "wrong_cloud": "no_cloud",
        "key": "my_default_key",
        "value": "my_default_value"
    })

    def run(self, command):
        command = command.format(**self.data)
        banner(command, c="-")
        print(command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return str(result)

    def setup(self):
        pass

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_001(self):
        HEADING("OS API")

        cloudname = 'kilo'
        d = ConfigDict("cloudmesh.yaml")
        cloud_details = d["cloudmesh"]["clouds"][cloudname]

        cp = CloudProviderOpenstackAPI(cloudname, cloud_details)

        #pprint(cp.list_flavor(cloudname))

        # pprint(cp.list_image(cloudname))


        pprint(cp.list_vm(cloudname))

        #pprint(cp.list_quota(cloudname))




from __future__ import print_function
from cloudmesh_client.shell.command import command
from cloudmesh_client.shell.console import Console
from cloudmesh_client.cloud.quota import Quota
from cloudmesh_client.default import Default
from cloudmesh_client.shell.command import PluginCommand, CloudPluginCommand


class QuotaCommand(PluginCommand, CloudPluginCommand):
    topics = {"quota": "cloud"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command quota")

    # noinspection PyUnusedLocal
    @command
    def do_quota(self, args, arguments):
        """
        ::

            Usage:
                quota list [--cloud=CLOUD] [--tenant=TENANT] [--format=FORMAT]

                Prints quota limit on a current project/tenant

            Options:
               --format=FORMAT  the output format [default: table]
               --cloud=CLOUD    the cloud name
               --tenant=TENANT  the tenant id

            Examples:
                cm quota list
                cm quota list --cloud=india --format=csv

        """
        if arguments["list"]:
            cloud = arguments["--cloud"] or Default.cloud

            if not cloud:
                Console.error("Default cloud doesn't exist")
                return
            tenant = arguments["--tenant"]
            output_format = arguments["--format"]
            list_quotas = Quota.list(cloud,
                                     tenant,
                                     output=output_format)
            Console.msg(list_quotas)
            return




""" run with

python setup.py install; nosetests -v --nocapture  tests/cm_cloud/test_register.py:Test_register.test_001

nosetests -v --nocapture tests/test_register.py

or

nosetests -v tests/test_register.py
"""

from cloudmesh_client.common.util import banner
from cloudmesh_client.common.util import HEADING

from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.default import Default

class Test_register:
    """
    # tests
    # for india only
    """
    data = dotdict({
        # "cloud": Default.cloud,
        # only retrieving for india kilo
        "cloud": "kilo",
        "group": "test",
        "vm": "testvm",
        "flavor": "TBD",
        "image": "TBD",
        "wrong_cloud": "no_cloud",
        "cert": "~/.cloudmesh/clouds/india/{cloud}/cacert.pem",
        "yaml": "cloudmesh1.yaml"
    })
    data.image = Default.get_image()
    data.flavor = Default.get_flavor()


    def run(self, command):
        command = command.format(**self.data)
        banner(command, c ="-")
        print (command)
        parameter = command.split(" ")
        shell_command = parameter[0]
        args = parameter[1:]
        result = Shell.execute(shell_command, args)
        print(result)
        return str(result)

    def setup(self):
        pass

    def tearDown(self):
        pass

    def test_001(self):
        
        HEADING("cm register remote {cloud}".format(**self.data))
        # os.sytem("yes | cm register india")
        result = self.run("cm register remote {cloud}")
        print(result)
        #result = Shell.cm("register", "india", "--force")
        assert "{cloud}".format(**self.data) in result
        assert "OS_AUTH_URL" in result
        assert "https://kilo.futuresystems.org:5000/v3" in result

    def test_002(self):
        HEADING("testing cm register random1")
        result = self.run ("cm register remote doesnotexist")
        assert "ERROR: executing" in result

    def test_003(self):
        HEADING("testing cm register list")
        result = self.run ("cm register list")
        assert "{cloud}".format(**self.data) in result


    def test_004(self):
        HEADING("testing cm register info")
        result = self.run("cm register info")
        assert "exists" in result

    def test_005(self):
        HEADING("testing cm register cat")
        result = self.run("cm register cat")
        assert "version:" in result

    def test_006(self):
        HEADING("testing cm register cat --yaml={yaml}".format(**self.data))
        result = self.run("cm register cat --yaml={yaml}")
        assert "ERROR: executing command" in result

    def test_007(self):
        HEADING("testing cm register edit --yaml=cloudmesh_doesnotexist.yaml")
        result = self.run("cm register edit --yaml=cloudmesh_doesnotexit.yaml")
        assert "ERROR: executing command" in result

    def test_008(self):
        HEADING("testing cm register list ssh")
        result = self.run("cm register list ssh")
        assert "| host" in result

    def test_009(self):
        HEADING("testing cm register remote")
        result = self.run("cm register remote")
        assert "Reading rc file from" in result

    def test_010(self):
        HEADING("cm register json {cloud}".format(**self.data))
        result = self.run ("cm register json {cloud}")
        assert "openstack" in result

    def test_011(self):
        HEADING("testing cm register json hadoop")
        result = self.run ("cm register json hadoop")
        assert "Cloud hadoop is not described in cloudmesh.yaml" in result



#
# demo to start os command
#
# from subprocess import check_output

# cmd = r'C:\cygwin64\bin\ps.exe'
# output = check_output(cmd)
# print (output)

from __future__ import print_function

from pipes import quote
import errno
import glob
import os
import os.path
import platform
import subprocess
import zipfile
from cloudmesh_client.shell.console import Console
from cloudmesh_client.common.util import path_expand


class SubprocessError(Exception):
    def __init__(self, cmd, returncode, stderr, stdout):
        self.cmd = cmd
        self.returncode = returncode
        self.stderr = stderr
        self.stdout = stdout


    def __str__(self):

        def indent(lines, amount, ch=' '):
            padding = amount * ch
            return padding + ('\n'+padding).join(lines.split('\n'))

        cmd = ' '.join(map(quote, self.cmd))
        s = ''
        s += 'Command: %s\n' % cmd
        s += 'Exit code: %s\n' % self.returncode

        if self.stderr:
            s += 'Stderr:\n' + indent(self.stderr, 4)
        if self.stdout:
            s += 'Stdout:\n' + indent(self.stdout, 4)

        return s


class Subprocess(object):

    def __init__(self, cmd, cwd=None, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=None):

        Console.debug_msg('Running cmd: {}'.format(' '.join(map(quote, cmd))))

        proc = subprocess.Popen(cmd, stderr=stderr, stdout=stdout, cwd=cwd, env=env)
        stdout, stderr = proc.communicate()

        self.returncode = proc.returncode
        self.stderr = stderr
        self.stdout = stdout

        if self.returncode != 0:
            raise SubprocessError(cmd, self.returncode, self.stderr, self.stdout)



class Shell(object):
    cygwin_path = 'bin'  # i copied fom C:\cygwin\bin

    command = {
        'windows': {},
        'linux': {},
        'darwin': {}
    }

    '''

    big question for badi and others

    how do we now define dynamically functions based on a list that we want to support

    what we want is where args are multiple unlimited parameters to the function

    def f(args...):
        name = get the name from f
        a = list of args...

        cls.execute(cmd, arguments=a, capture=True, verbose=False)

    commands = ['ps', 'ls', ..... ]
    for c in commands:
        generate this command and add to this class dynamically

    or do something more simple

    ls = cls.execute('cmd', args...)

    '''

    @classmethod
    def check_output(cls, *args, **kwargs):
        """Thin wrapper around :func:`subprocess.check_output`
        """
        return subprocess.check_output(*args, **kwargs)


    @classmethod
    def ls(cls, *args):
        return cls.execute('ls', args)

    @classmethod
    def ps(cls, *args):
        return cls.execute('ps', args)

    @classmethod
    def bash(cls, *args):
        return cls.execute('bash', args)

    @classmethod
    def cat(cls, *args):
        return cls.execute('cat', args)

    @classmethod
    def git(cls, *args):
        return cls.execute('git', args)

    @classmethod
    def VBoxManage(cls, *args):
        return cls.execute('VBoxManage', args)

    @classmethod
    def blockdiag(cls, *args):
        return cls.execute('blockdiag', args)

    @classmethod
    def cm(cls, *args):
        return cls.execute('cm', args)

    @classmethod
    def fgmetric(cls, *args):
        return cls.execute('fgmetric', args)

    @classmethod
    def fgrep(cls, *args):
        return cls.execute('fgrep', args)

    @classmethod
    def gchproject(cls, *args):
        return cls.execute('gchproject', args)

    @classmethod
    def gchuser(cls, *args):
        return cls.execute('gchuser', args)

    @classmethod
    def glusers(cls, *args):
        return cls.execute('glusers', args)

    @classmethod
    def gmkproject(cls, *args):
        return cls.execute('gmkproject', args)

    @classmethod
    def grep(cls, *args):
        return cls.execute('grep', args)

    @classmethod
    def gstatement(cls, *args):
        return cls.execute('gstatement', args)

    @classmethod
    def head(cls, *args):
        return cls.execute('head', args)

    @classmethod
    def keystone(cls, *args):
        return cls.execute('keystone', args)

    @classmethod
    def kill(cls, *args):
        return cls.execute('kill', args)

    @classmethod
    def ls(cls, *args):
        return cls.execute('ls', args)

    @classmethod
    def mongoimport(cls, *args):
        return cls.execute('mongoimport', args)

    @classmethod
    def mysql(cls, *args):
        return cls.execute('mysql', args)

    @classmethod
    def nosetests(cls, *args):
        return cls.execute('nosetests', args)

    @classmethod
    def nova(cls, *args):
        return cls.execute('nova', args)

    @classmethod
    def ping(cls, host=None, count=1):
        return cls.execute('ping', "-c {} {}".format(count, host))

    @classmethod
    def pwd(cls, *args):
        return cls.execute('pwd', args)

    @classmethod
    def rackdiag(cls, *args):
        return cls.execute('rackdiag', args)

    @classmethod
    def rm(cls, *args):
        return cls.execute('rm', args)

    @classmethod
    def rsync(cls, *args):
        return cls.execute('rsync', args)

    @classmethod
    def scp(cls, *args):
        return cls.execute('scp', args)

    @classmethod
    def sort(cls, *args):
        return cls.execute('sort', args)

    @classmethod
    def sh(cls, *args):
        return cls.execute('sh', args)

    @classmethod
    def ssh(cls, *args):
        return cls.execute('ssh', args)

    @classmethod
    def sudo(cls, *args):
        return cls.execute('sudo', args)

    @classmethod
    def tail(cls, *args):
        return cls.execute('tail', args)

    @classmethod
    def vagrant(cls, *args):
        return cls.execute('vagrant', args)

    @classmethod
    def mongod(cls, *args):
        return cls.execute('mongod', args)

    @classmethod
    def grep(cls, *args):
        return cls.execute('grep', args)

    @classmethod
    def dialog(cls, *args):
        return cls.execute('dialog', args)

    @classmethod
    def pip(cls, *args):
        return cls.execute('pip', args)

    @classmethod
    def remove_line_with(cls, lines, what):
        result = []
        for line in lines:
            if what not in line:
                result = result + [line]
        return result

    @classmethod
    def find_lines_with(cls, lines, what):
        result = []
        for line in lines:
            if what in line:
                result = result + [line]
        return result

    def __init__(cls):
        if cls.operating_system() == "windows":
            cls.find_cygwin_executables()
        else:
            pass
            # implement for cmd, for linux we can just pass as it includes everything

    @classmethod
    def find_cygwin_executables(cls):
        """
        find the executables 
        """
        exe_paths = glob.glob(cls.cygwin_path + r'\*.exe')
        # print cls.cygwin_path
        # list all *.exe in  cygwin path, use glob
        for c in exe_paths:
            exe = c.split('\\')
            name = exe[1].split('.')[0]
            # command['windows'][name] = "{:}\{:}.exe".format(cygwin_path, c)
            cls.command['windows'][name] = c

    @classmethod
    def terminal_type(cls):
        """
        returns  darwin, cygwin, cmd, or linux
        """
        what = platform.system().lower()

        kind = 'UNDEFINED_TERMINAL_TYPE'
        if 'linux' in what:
            kind = 'linux'
        elif 'darwin' in what:
            kind = 'darwin'
        elif 'cygwin' in what:
            kind = 'cygwin'
        elif 'windows' in what:
            kind = 'windows'

        return kind

    @classmethod
    def which(cls, command):
        t = cls.ttype()
        if 'windows' in t and cls.command_exists(name):
            return cls.command['windows'][name]
        elif 'linux' in t:
            cmd = ["which", command]
            result = subprocess.check_output(cmd).rstrip()
            if len(result) == 0:
                return None
            else:
                return result

    @classmethod
    def command_exists(cls, name):
        t = cls.ttype()
        if 'windows' in t:
            # only for windows
            cls.find_cygwin_executables()
            return name in cls.command['windows']
        elif 'linux' in t:
            r = which(name)
            return r

    @classmethod
    def list_commands(cls):
        t = cls.ttype()
        if 'windows' in t:
            # only for windows
            cls.find_cygwin_executables()
            print('\n'.join(cls.command['windows']))
        else:
            print("ERROR: this command is not supported for this OS")

    @classmethod
    def operating_system(cls):
        return platform.system().lower()

    @classmethod
    def execute(cls, cmd, arguments="", shell=False, cwd=None, traceflag=True, witherror=True):
        """Run Shell command

        :param cmd: command to run
        :param arguments: we dont know yet
        :param capture: if true returns the output
        :return:
        """
        # print "--------------"
        result = None
        terminal = cls.terminal_type()
        # print cls.command
        os_command = [cmd]
        if terminal in ['linux', 'windows']:
            os_command = [cmd]
        elif 'cygwin' in terminal:
            if not cls.command_exists(cmd):
                print("ERROR: the command could not be found", cmd)
                return
            else:
                os_command = [cls.command[cls.operating_system()][cmd]]

        if isinstance(arguments, list):
            os_command = os_command + arguments
        elif isinstance(arguments, tuple):
            os_command = os_command + list(arguments)
        elif isinstance(arguments, str):
            os_command = os_command + arguments.split()
        else:
            print("ERROR: Wrong parameter type", type(arguments))

        if cwd is None:
            cwd = os.getcwd()
        try:
            if shell:
                result = subprocess.check_output(
                    os_command,
                    stderr=subprocess.STDOUT,
                    shell=True,
                    cwd=cwd)
            else:
                result = subprocess.check_output(
                    os_command,
                    # shell=True,
                    stderr=subprocess.STDOUT,
                    cwd=cwd)
        except:
            if witherror:
                Console.error("problem executing subprocess", traceflag=traceflag)
        if result is not None:
            result = result.strip().decode()
        return result

    @classmethod
    def mkdir(cls, newdir):
        """works the way a good mkdir should :)
        - already exists, silently complete
        - regular file in the way, raise an exception
        - parent directory(ies) does not exist, make them as well
        """
        """http://code.activestate.com/recipes/82465-a-friendly-mkdir/"""

        newdir = path_expand(newdir)
        try:
            os.makedirs(newdir)
        except OSError as e:

            # EEXIST (errno 17) occurs under two conditions when the path exists:
            # - it is a file
            # - it is a directory
            #
            # if it is a file, this is a valid error, otherwise, all
            # is fine.
            if e.errno == errno.EEXIST and os.path.isdir(newdir):
                pass
            else: raise


    def unzip(source_filename, dest_dir):
        """
        unzips a file into the destination directory
        :param dest_dir: the destination directory
        :return:
        """

        with zipfile.ZipFile(source_filename) as zf:
            for member in zf.infolist():
                # Path traversal defense copied from
                # http://hg.python.org/cpython/file/tip/Lib/http/server.py#l789
                words = member.filename.split('/')
                path = path_expand(dest_dir)
                for word in words[:-1]:
                    drive, word = os.path.splitdrive(word)
                    head, word = os.path.split(word)
                    if word in (os.curdir, os.pardir, ''):
                        continue
                    path = os.path.join(path, word)
                zf.extract(member, path)


def main():
    shell = Shell()

    print(shell.terminal_type())

    r = shell.execute('pwd')  # copy line replace
    print(r)

    # shell.list()

    # print json.dumps(shell.command, indent=4)

    # test some commands without args
    """
    for cmd in ['whoami', 'pwd']:
        r = shell._execute(cmd)
        print ("---------------------")
        print ("Command: {:}".format(cmd))
        print ("{:}".format(r))
        print ("---------------------")
    """
    r = shell.execute('ls', ["-l", "-a"])
    print(r)

    r = shell.execute('ls', "-l -a")
    print(r)

    r = shell.ls("-aux")
    print(r)

    r = shell.ls("-a", "-u", "-x")
    print(r)

    r = shell.pwd()
    print(r)


if __name__ == "__main__":
    main()



from ..CloudmeshDatabase import CloudmeshDatabase, CloudmeshMixin, CloudmeshVMMixin
from sqlalchemy import Column, Date, Integer, String


# noinspection PyPep8Naming
class IMAGE_LIBCLOUD(CloudmeshMixin, CloudmeshDatabase.Base):
    __tablename__ = "image_libcloud"

    __kind__ = 'image'
    __provider__ = "libcloud"
    __mergefields__ = ["username"]

    username = Column(String)
    uuid = Column(String)
    status = Column(String)
    updated = Column(String)
    created = Column(String)
    architecture = Column(String)
    description = Column(String)
    hypervisor = Column(String)
    image_id = Column(String)
    image_location = Column(String)
    image_type = Column(String)
    is_public = Column(String)
    kernel_id = Column(String)
    owner_alias = Column(String)
    owner_id = Column(String)
    platform = Column(String)
    ramdisk_id = Column(String)
    state = Column(String)
    virtualization_type = Column(String)

    def __init__(self,
                 **kwargs):
        super(IMAGE_LIBCLOUD, self).set_defaults(**kwargs)

        self.status = kwargs.get('status')
        self.architecture = kwargs.get('architecture')
        self.description = kwargs.get('description')
        self.hypervisor = kwargs.get('hypervisor')
        self.image_id = kwargs.get('image_id')
        self.image_location = kwargs.get('image_location')
        self.image_type = kwargs.get('image_type')
        self.is_public = kwargs.get('is_public')
        self.kernel_id = kwargs.get('kernel_id')
        self.owner_alias = kwargs.get('owner_alias')
        self.owner_id = kwargs.get('owner_id')
        self.platform = kwargs.get('platform')
        self.ramdisk_id = kwargs.get('ramdisk_id')
        self.state = kwargs.get('state')


# noinspection PyPep8Naming
class FLAVOR_LIBCLOUD(CloudmeshMixin, CloudmeshDatabase.Base):
    __tablename__ = "flavor_libcloud"

    __kind__ = 'flavor'
    __provider__ = "libcloud"

    uuid = Column(String)
    flavor_id = Column(String)
    ram = Column(String)
    disk = Column(String)
    bandwidth = Column(String)
    price = Column(String)
    cpu = Column(String)

    def __init__(self,
                 **kwargs):
        super(FLAVOR_LIBCLOUD, self).set_defaults(**kwargs)

        self.uuid = kwargs.get("uuid")
        self.flavor_id = kwargs.get("flavor_id")
        self.ram = kwargs.get("ram")
        self.disk = kwargs.get("disk")
        self.bandwidth = kwargs.get("bandwidth")
        self.price = kwargs.get("price")
        self.cpu = kwargs.get("cpu")


# noinspection PyPep8Naming
class VM_LIBCLOUD(CloudmeshMixin, CloudmeshVMMixin, CloudmeshDatabase.Base):
    __tablename__ = "vm_libcloud"

    __kind__ = 'vm'
    __provider__ = "libcloud"
    __mergefields__ = ["username"]

    username = Column(String)
    uuid = Column(String)
    state = Column(String)
    public_ips = Column(String)
    private_ips = Column(String)
    image_name = Column(String)
    availability = Column(String)
    image_id = Column(String)
    instance_id = Column(String)
    instance_type = Column(String)
    key = Column(String)
    private_dns = Column(String)
    root_device_name = Column(String)
    root_device_type = Column(String)
    status = Column(String)
    image = Column(String)
    flavor = Column(String)
    group = Column(String)

    def __init__(self, **kwargs):
        super(VM_LIBCLOUD, self).set_defaults(**kwargs)

        self.uuid = kwargs.get("node_id")
        self.state = kwargs.get("state")
        self.public_ips = kwargs.get("public_ips")
        self.private_ips = kwargs.get("private_ips")
        self.image_name = kwargs.get("image_name")
        self.availability = kwargs.get("availability")
        self.image_id = kwargs.get("image_id")
        self.instance_id = kwargs.get("instance_id")
        self.instance_type = kwargs.get("instance_type")
        self.private_dns = kwargs.get("private_dns")
        self.root_device_name = kwargs.get("root_device_name")
        self.root_device_type = kwargs.get("root_device_type")
        self.status = kwargs.get("status", "defined")
        self.key = kwargs.get("key", None)
        self.group = kwargs.get("group", None)



from cloudmesh_client.common.Shell import Shell


# noinspection PyBroadException
class Experiment(object):
    @classmethod
    def rm(cls, cluster, id=None, format=None):
        data = {
            "CLUSTER": cluster,
            "ID": id,
        }
        result = None
        if id is not None:
            try:
                result = Shell.ssh(cluster, "rm -rf experiment/{ID}".format(**data))
            except Exception as e:
                pass
        else:
            try:
                result = Shell.ssh(cluster, "rm -rf experiment/*").split("\n")
            except Exception as e:
                pass

        return result

    @classmethod
    def list(cls, cluster, id=None, format=None):
        data = {
            "CLUSTER": cluster,
            "ID": id,
        }
        result = None
        if id is not None:
            try:
                result = Shell.ssh(cluster, "ls experiment/{ID}".format(**data))
                result = result.split("\n")
            except Exception as e:
                result = None
        else:
            try:
                result = Shell.ssh(cluster, "ls experiment").split("\n")
                ids = sorted([int(i) for i in result])
                if format not in [None, 'txt']:
                    result = ids
            except Exception as e:
                result = None

        return result

    @classmethod
    def output(cls, cluster, id=None, format=None):
        data = {
            "CLUSTER": cluster,
            "ID": id,
        }
        result = None
        if id is None:
            ids = list(cluster)
        else:
            ids = [id]
        result = []
        for i in ids:
            try:
                result.append(Shell.ssh(cluster, "cat experiment/{}/*.out".format(i)))
            except:
                result.append("")
        # if result == []:
        #    result = None
        return result



import json
import subprocess
import textwrap

from jinja2 import Template

from cloudmesh_client.common.util import path_expand


class Rack(object):
    def __init__(self, servers):
        self.template_string = None
        self.servers = servers
        self.diag = None
        self.data = None

    def set_template(self, template):

        if template is None:
            self.template_string = "rackdiag {" + '''
              // Change order of rack-number as ascending
              ascending;

              // define height of rack
              20U;

              // define description of rack
              description = "Comet Cluster Virtualization Rack";

              // define rack units
              1: UPS [2U];   // define height of unit
              3: Network [color=green]
              {% for x in range (0, 10) %}
              {{x+4}}: Server [''' + " ".join([
                '{{server[x]["label"]}}',
                '{{server[x]["numbered"]}}',
                '{{server[x]["fontsize"]}}',
                '{{server[x]["shape"]}}',
                '{{server[x]["textcolor"]}}',
                '{{server[x]["color"]}}'
            ]) + "]" + '''{% endfor %}''' + "\n}"
        else:
            self.template_string = template
        self.generate_rack(self.servers)

    def generate_rack(self, servers):
        empty = {
            "color": 'color="white" ',
            "label": "",
            "numbered": "",
            "fontsize": "",
            "shape": "",
            "textcolor": "",
        }
        self.data = []
        for server in range(0, servers):
            self.data.append(dict(empty))

    def __str__(self):
        return json.dumps(self.data, indent=4)

    def dump(self):
        content = self.diag
        result = []
        counter = 1
        for line in content.split("\n"):
            result.append("{0:>5}: {1}".format(counter, line))
            counter += 1
        return '\n'.join(result)

    def set(self, server, **kwargs):
        for attribute in kwargs:
            comma = ' '
            if attribute is not 'color':
                comma = ', '
            self.data[server - 1][attribute] = '{}="{}"{}' \
                .format(attribute, kwargs[attribute], comma)

    def set_color(self, server, color):
        self.set(server, color=color)

    def set_label(self, server, label):
        self.set(server, label=label)

    def set_numbering(self, server, numbering):
        self.set(server, numbering=numbering)

    def set_fontsize(self, server, fontsize):
        self.set(server, fontsize=fontsize)

    def set_textcolorl(self, server, textcolorl):
        self.set(server, textcolorl=textcolorl)

    def set_shape(self, server, shape):
        self.set(server, shape=shape)

    def render(self):
        template = Template(textwrap.dedent(self.template_string))
        self.diag = template.render(server=self.data)
        return self.diag

    def diagram(self, name):
        with open('{}.diag'.format(path_expand(name)), 'w') as f:
            f.write(self.diag)

    def svg(self, name):
        self.diagram(name)
        cmd = ['rackdiag', "-T", "svg", "{}.diag".format(path_expand(name))]
        subprocess.Popen(cmd).wait()

    def view(self, name):
        cmd = ['open', "{}.svg".format(path_expand(name))]
        subprocess.Popen(cmd)


if __name__ == "__main__":
    rack = Rack(10)
    rack.set_template(None)
    rack.set(10, color="blue")
    rack.set(9, color="green")
    rack.set(8, textcolor="red")
    rack.set(7, textsize="24")
    rack.set(7, shape="cloud")
    rack.set(6, numbered="1")

    result = rack.render()
    # print rack.dump()

    name = "~/.cloudmesh/c"
    rack.svg(name)
    rack.view(name)



from __future__ import print_function

import os

from cloudmesh_client.common.dotdict import dotdict
from cloudmesh_client.common.util import banner
from cloudmesh_client.shell.command import command, PluginCommand, CloudPluginCommand


class TestCommand(PluginCommand, CloudPluginCommand):
    topics = {"test": "cloud"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command test")

    # noinspection PyUnusedLocal
    @command
    def do_test(self, args, arguments):
        """
        ::

          Usage:
             test list
             test [TEST] [NUMBER] [--test=TESTER]

          This is an internal command and is ment for developers. If executed, in the source directory,
          it will execute the specified series of tests in the tests directory.
          managing the admins test test test test

          Arguments:
            TEST     the name of the test directory
            NUMBER   the number of a specific test

          Options:
            --test=TESTER   nosetests or py.test. [default: nosetests -v --nocapture]

          Examples:
              cm test var
                  finds the first test that contains var and than executes all tests in it

              cm test var 4
                  finds the first test that contains var and than executes the test with number 004

        """

        data = dotdict({
            'test': arguments["TEST"],
            'number': arguments["NUMBER"],
            "tester": arguments["--test"]
        })

        def get_file():
            data.file = None
            for file in files:
                if data.test in file:
                    break
            data.file = file

        def run(command):
            command = command.format(**data)
            banner(command, c="-")
            parameter = command.split(" ")

            os.system(command)

            # shell_command = parameter[0]
            # args = parameter[1:]

            # result = Shell.execute(shell_command, args)
            # print(result)
            # return str(result)

        # command = "nosetests -v  --nocapture {0}:{1}.{2}".format(filename, classname, name)

        files = []
        dirs = []
        for root, dirnames, filenames in os.walk('tests'):
            for filename in filenames:
                if ".pyc" in filename:
                    continue
                if ".py" not in filename:
                    dirs.append(os.path.join(root, filename))
                elif filename not in ["__init__.py"]:
                    files.append(os.path.join(root, filename))

        if arguments["list"]:
            print('\n'.join(files))

            # find tests
            return ""

        if data.test is None:
            command = "{tester}"

        elif data.test in dirs:
            command = "{tester} {test}"

        elif data.number is None:
            get_file()
            command = "{tester} {file}"
            # run all in that file
        else:
            # run specific test
            get_file()
            # python setup.py install; nosetests -v --nocapture  tests/cm_cloud/test_group.py:Test_group.test_001
            data.basename = os.path.basename(data.file).replace(".py", "")
            data.basename = data.basename.replace("test_", "")
            data.number = data.number.zfill(3)

            command = "{tester} {file}:Test_{basename}.test_{number}"

        run(command)

        return ""



from cloudmesh_client.shell.plugins.ColorCommand import ColorCommand
from cloudmesh_client.shell.plugins.CloudCommand import CloudCommand
from cloudmesh_client.shell.plugins.CometCommand import CometCommand
from cloudmesh_client.shell.plugins.DefaultCommand import DefaultCommand
from cloudmesh_client.shell.plugins.FlavorCommand import FlavorCommand
from cloudmesh_client.shell.plugins.GroupCommand import GroupCommand
from cloudmesh_client.shell.plugins.HpcCommand import HpcCommand
from cloudmesh_client.shell.plugins.ImageCommand import ImageCommand
from cloudmesh_client.shell.plugins.InventoryCommand import InventoryCommand
from cloudmesh_client.shell.plugins.AkeyCommand import AkeyCommand
from cloudmesh_client.shell.plugins.KeyCommand import KeyCommand
from cloudmesh_client.shell.plugins.LimitsCommand import LimitsCommand
from cloudmesh_client.shell.plugins.ListCommand import ListCommand
from cloudmesh_client.shell.plugins.ManCommand import ManCommand
from cloudmesh_client.shell.plugins.NetworkCommand import NetworkCommand
from cloudmesh_client.shell.plugins.NovaCommand import NovaCommand
from cloudmesh_client.shell.plugins.OpenCommand import OpenCommand
from cloudmesh_client.shell.plugins.QuotaCommand import QuotaCommand
from cloudmesh_client.shell.plugins.RegisterCommand import RegisterCommand
from cloudmesh_client.shell.plugins.ReservationCommand import ReservationCommand
from cloudmesh_client.shell.plugins.ResetCommand import ResetCommand
from cloudmesh_client.shell.plugins.SecgroupCommand import SecgroupCommand
from cloudmesh_client.shell.plugins.SecureShellCommand import SecureShellCommand
from cloudmesh_client.shell.plugins.SelectCommand import SelectCommand
from cloudmesh_client.shell.plugins.ServerCommand import ServerCommand
from cloudmesh_client.shell.plugins.SubmitCommands import SubmitCommands
from cloudmesh_client.shell.plugins.TerminalCommands import TerminalCommands
from cloudmesh_client.shell.plugins.UsageCommand import UsageCommand
from cloudmesh_client.shell.plugins.VmCommand import VmCommand
from cloudmesh_client.shell.plugins.PyCommand import PyCommand
from cloudmesh_client.shell.plugins.LauncherCommand import LauncherCommand
from cloudmesh_client.shell.plugins.PortalCommand import PortalCommand
from cloudmesh_client.shell.plugins.SyncCommand import SyncCommand
from cloudmesh_client.shell.plugins.CheckCommand import CheckCommand
from cloudmesh_client.shell.plugins.DebugCommand import DebugCommand
from cloudmesh_client.shell.plugins.RefreshCommand import RefreshCommand
from cloudmesh_client.shell.plugins.ClusterCommand import ClusterCommand
from cloudmesh_client.shell.plugins.TimerCommand import TimerCommand
from cloudmesh_client.shell.plugins.LoadCommand import LoadCommand
from cloudmesh_client.shell.plugins.TestCommand import TestCommand
from cloudmesh_client.shell.plugins.InfoCommand import InfoCommand
from cloudmesh_client.shell.plugins.WorkflowCommand import WorkflowCommand
from cloudmesh_client.shell.plugins.VcCommand import VcCommand
from cloudmesh_client.shell.plugins.SetupCommand import SetupCommand
from cloudmesh_client.shell.plugins.StackCommand import StackCommand
from cloudmesh_client.shell.plugins.ClusterCommand2 import Cluster2Command
from cloudmesh_client.shell.plugins.HadoopCommand import HadoopCommand
from cloudmesh_client.shell.plugins.DeployCommand import DeployCommand

# from cloudmesh_client.shell.plugins.LogLevelCommand import LogLevelCommand
# from cloudmesh_client.shell.plugins.WhoCommand .import WhoCommand





""" run with

python setup.py install; nosetests -v --nocapture tests/cm_basic/test_tables.py:Test_tables.test_001

nosetests -v --nocapture tests/cm_basic/test_tables.py

or

nosetests -v tests/cm_basic/test_tables.py

"""

from pprint import pprint

from cloudmesh_client.common.Printer import Printer
from cloudmesh_client.common.util import HEADING


# noinspection PyPep8Naming
class Test_tables:
    """define tests for dict printer so you test
    yaml
    json
    table
    csv
    dict
    printing
    """

    def setup(self):
        self.d = [
            {
                "id": "a",
                "x": 1,
                "y": 2,
            },
            {
                "id": "b",
                "x": 3,
                "y": 4,
            },
        ]

    # noinspection PyPep8Naming
    def tearDown(self):
        pass

    def test_001_yaml(self):
        HEADING("Printer.write of a yaml object")
        output = Printer.write(self.d, order=None, header=None, output="yaml", sort_keys=True)
        print(output)
        assert ":" in output

    def test_002_json(self):
        HEADING("Printer.write of a json object")
        output = Printer.write(self.d, order=None, header=None, output="json", sort_keys=True)
        print(output)
        assert "{" in output

    def test_003_dict(self):
        HEADING("Printer.write of a dict object")
        output = Printer.write(self.d, order=None, header=None, output="dict", sort_keys=True)
        pprint(output)
        assert type(output) == dict

    def test_004_table(self):
        HEADING("Printer.write of a table object")
        output = Printer.write(self.d, order=None, header=None, output="table", sort_keys=True)
        print(output)
        # assert "id" in str(output)

    def test_005_csv(self):
        HEADING("Printer.write of a csv object")
        output = Printer.write(self.d, order=None, header=None, output="csv", sort_keys=True)
        print(output)
        # assert "id" in str(output)
